;**************************************************************************************************
;*
;* Framewrk.inc: Code framework for the NGPC.
;*
;* Author      : Steve Robb
;*
;**************************************************************************************************

	IfNDef FRAMEWRK_INC
FRAMEWRK_INC Equ TRUE


;**********************
;* Assembler settings *
;**********************

	Cpu     93C141
	MaxMode on
	Org     200000h


;*****************
;* Include files *
;*****************

	Include "Assert.inc"
	Include "Answer.inc"
	Include "Version.inc"

	Include "stddef96.inc"


;*************
;* Functions *
;*************

FRGB    Function nR, nG, nB, (((nB) >> 4) << 8) + (((nG) >> 4) << 4) + ((nR) >> 4)

Trunc   Function nValue, Int((nValue) - (Sgn(nValue) * (Abs(nValue) - Int(Abs(nValue)))))

NonZero Function nValue, Abs(Sgn(nValue))


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Quote
;* Parameters  - any
;* Returns     - void
;* Description - Compiles the specified arguments.  Used to trick the preprocessor.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Quote: Macro

	;/* Output all arguments */
	ALLARGS

	EndM

;****************
;* End of Quote *
;****************


;**************************************************************************************************
;*
;* Macro       - For
;* Parameters  - forvariable: The name of the variable being looped.
;*               nStart     : The start value of the variable.
;*               nEnd       : The end value of the variable.
;*               nStep      : The value to step by (optional).
;* Returns     - void
;* Description - Compile-time "for" loop.
;* Author      - Steve Robb
;*
;**************************************************************************************************

For: Macro forvariable, nStart, nEnd, nStep

	;/* Validate parameters */
	StaticAssert_NotBlank forvariable, "Syntax: for <variable name>, <start>, <end>[, <step>]"
	StaticAssert_NotBlank      nStart, "Syntax: for <variable name>, <start>, <end>[, <step>]"
	StaticAssert_NotBlank        nEnd, "Syntax: for <variable name>, <start>, <end>[, <step>]"

	;/* Check if step was not specified */
	IfB nStep

		;/* Set step value to direction from nStart to nEnd */
istep_forvariable Eval Sgn((nEnd) - (nStart))

	;/* A step was specified */
	Else

		;/* Set step value to specified step */
istep_forvariable Eval nStep

	EndIf

	;/* Check if start and end are equal */
	If (nStart) == (nEnd)

		;/* Repeat once */
irepeat_forvariable Eval 1

	;/* Start and end are not equal */
	Else

		;/* Repeat for required number of steps + 1 */
irepeat_forvariable Eval Trunc(((nEnd) - (nStart)) / istep_forvariable) + 1

	EndIf

	;/* Set variable value to one step behind start */
forvariable Eval (nStart) - istep_forvariable

	;/* Repeat for required number of steps + 1 */
	Quote Rept irepeat_forvariable

	;/* Increment variable by step */
forvariable Eval forvariable + istep_forvariable

	EndM

;**************
;* End of For *
;**************


;**************
;* ROM header *
;**************

	DB " LICENSED BY SNK CORPORATION" ;/* License string                     */
	DD EntryPoint                     ;/* Start of execution                 */
	DW 0                              ;/* Cartridge ID                       */
	DB 0                              ;/* Cartridge version                  */
	DB 10h                            ;/* Format code (NGP = 0h, NGPC = 10h) */
	DB CART_TITLE_12_CHARS            ;/* Cartridge title                    */
	DD 0h                             ;/* Reserved                           */
	DD 0h                             ;/* Reserved                           */
	DD 0h                             ;/* Reserved                           */
	DD 0h                             ;/* Reserved                           */


;*********************
;* Interrupt vectors *
;*********************

InterruptVectors:

	DD Interrupt_SWI3         ;/* Software Interrupt (SWI 3)      */
	DD Interrupt_SWI4         ;/* Software Interrupt (SWI 4)      */
	DD Interrupt_SWI5         ;/* Software Interrupt (SWI 5)      */
	DD Interrupt_SWI6         ;/* Software Interrupt (SWI 6)      */
	DD Interrupt_Alarm        ;/* RTC Alarm Interrupt             */
	DD Interrupt_VSync        ;/* Vertical Blanking Interrupt     */
	DD Interrupt_Z80          ;/* Interrupt from Z80              */
	DD Interrupt_Timer0       ;/* Timer Interrupt (8 bit timer 0) */
	DD Interrupt_Timer1       ;/* Timer Interrupt (8 bit timer 1) */
	DD Interrupt_Timer2       ;/* Timer Interrupt (8 bit timer 2) */
	DD Interrupt_Timer3       ;/* Timer Interrupt (8 bit timer 3) */
	DD Interrupt_Transmission ;/* Serial Transmission Interrupt   */
	DD Interrupt_Reception    ;/* Serial Reception Interrupt      */
	DD Interrupt_Unused       ;/* (Reserved)                      */
	DD Interrupt_MicroDMA0    ;/* End Micro DMA Int (MicroDMA 0)  */
	DD Interrupt_MicroDMA1    ;/* End Micro DMA Int (MicroDMA 1)  */
	DD Interrupt_MicroDMA2    ;/* End Micro DMA Int (MicroDMA 2)  */
	DD Interrupt_MicroDMA3    ;/* End Micro DMA Int (MicroDMA 3)  */


;****************
;* System calls *
;****************

SYSTEM_CALL:
        push    sr
        ldf     3
        push    xix
        add     w,w
        add     w,w
        ld      xix,0fffe00h
        ld      xix,(xix+w)
        call    (xix)
        pop     xix
        pop     sr
        ret


;**************************************************************************************************
;*
;* Interrupt   - EntryPoint
;* Parameters  - none
;* Returns     - none
;* Description - NGPC code entry point.
;* Author      - Steve Robb
;*
;**************************************************************************************************

EntryPoint:

	;/* Apply operating system patch */
	cpb (AVersion_Register),                    0
	jr                   nz, EntryPoint_PatchExit
	res                   3,             (06F83h)
	ld             (06DA0h),                    0
EntryPoint_PatchExit:

	;/* Enable automatic inactivity shutdown */
	UserAnswer_EnableInactiveAutoShutdown

	;/* Disable bit 5 of User Answer, as per NGPC startup recommendations */
	UserAnswer_DisableBit5

	;/* Set up horizontal interrupt */
	andb  (TRUN), 10001110b
	ldb   (TMOD),       00h
	ldb  (TREG0),       01h
	ldb      rw3,         4
	ldb      rb3,       03h
	ldb      rc3,       02h
	swi        1
	orb   (TRUN), 00000001h

	;/* Install interrupt vectors */
	lda     xix,           (InterruptVectors)
	lda     xiy,                      (rSWI3)
	ldb       b,                           18
EntryPoint_InstallInterrupts:
	ld      xwa,                       (xix+)
	ld   (xiy+),                          xwa
	djnz      b, EntryPoint_InstallInterrupts

	;/* Enable interrupts */
	ei 0

	;/* Jump to main function */
	jp main

;*********************
;* End of EntryPoint *
;*********************


;**************************************************************************************************
;*
;* Interrupt   - Interrupt_Unused
;* Parameters  - none
;* Returns     - none
;* Description - Called when an unused interrupt occurs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Interrupt_Unused:

	;/* Return from interrupt */
	reti

;***************************
;* End of Interrupt_Unused *
;***************************


;**************************************************************************************************
;*
;* Function    - Function_Unused
;* Parameters  - none
;* Returns     - none
;* Description - Called when an unused function occurs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Function_Unused:

	;/* Return from function */
	ret

;**************************
;* End of Function_Unused *
;**************************


rUSERS  equ     6F85h   ; User Shutdown (Read only byte)

; User Program Interrupt Vectors

rSWI3   equ     6FB8h   ; Software Interrupt (SWI 3)
rSWI4   equ     6FBCh   ; Software Interrupt (SWI 4)
rSWI5   equ     6FC0h   ; Software Interrupt (SWI 5)
rSWI6   equ     6FC4h   ; Software Interrupt (SWI 6)
rRTCI   equ     6FC8h   ; RTC Alarm Interrupt
rVBI    equ     6FCCh   ; Vertical Blanking Interrupt
rZ80I   equ     6FD0h   ; Interrupt from Z80
rTI0    equ     6FD4h   ; Timer Interrupt (8 bit timer 0)
rTI1    equ     6FD8h   ; Timer Interrupt (8 bit timer 1)
rTI2    equ     6FDCh   ; Timer Interrupt (8 bit timer 2)
rTI3    equ     6FE0h   ; Timer Interrupt (8 bit timer 3)
rSTI    equ     6FE4h   ; Serial Transmission Interrupt
rSRI    equ     6FE8h   ; Serial Reception Interrupt
rEMDMA0 equ     6FF0h   ; End Micro DMA Int (MicroDMA 0)
rEMDMA1 equ     6FF4h   ; End Micro DMA Int (MicroDMA 1)
rEMDMA2 equ     6FF8h   ; End Micro DMA Int (MicroDMA 2)
rEMDMA3 equ     6FFCh   ; End Micro DMA Int (MicroDMA 3)

; RAM Equates

VECT_SHUTDOWN          equ     0       ; Shutdown (Power OFF)
VECT_CLOCKGEARSET      equ     1       ; CPU operation clock setting
VECT_INTLVSET          equ     4       ; Interrupt level setting
VECT_RTCGET            equ     2       ; Real time clock - obtain time
VECT_ALARMSET          equ     9       ; Real time clock - alarm setting during game operation
VECT_ALARMDOWNSET      equ     0bh     ; Real time clock - unit start up alarm setting
VECT_SYSFONTSET        equ     5       ; System font setting
VECT_FLASHWRITE        equ     6       ; Flash memory - data write
VECT_FLASHALLERS       equ     7       ; Flash memory - erase all blocks
VECT_FLASHERS          equ     8       ; Flash memory - erase specified blocks
VECT_FLASHPROTECT      equ     0dh     ; Flash memory - protect specified blocks
VECT_GEMODESET         equ     0eh     ; Color LCD color mode setting (color version only)


	EndIf ;// IfNDef FRAMEWRK_INC

;***********************
;* End of Framewrk.inc *
;***********************
