;**************************************************************************************************
;*
;* LinkUp.inc: Link-up macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef LINKUP_INC
LINKUP_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"
	Include "UserRAM.inc"


;******************
;* Public defines *
;******************

	IfDef _FAKELINKUP

		If (_FAKELINKUP < 1) || (_FAKELINKUP > 3) || (Int(_FAKELINKUP) <> _FAKELINKUP)

			Error "LinkUp.inc: _FAKELINKUP must be 1, 2 or 1+2"

		EndIf

	EndIf


;**************************************************************************************************
;*
;* Notes: Contrary to the examples in the documentation, the description of the 
;*        VECT_COMSENDSTATUS bios call is correct, i.e. the lower-order byte contains the
;*        number of elements in the buffer, and the higher-order byte contains the error
;*        flags.  The error flags *must* be masked off as the reserved bits are not guaranteed
;*        to be zero.
;*
;*        After VECT_SENDSTART is called, the buffer will appear to decrease in size by 1 byte,
;*        whether or not the transfer has been successful.  I assume this is due to the byte
;*        being placed in a further buffer for transfer.  Further calls will not be successful
;*        until the byte has between received by the other NGPC.
;*
;*        If link-up code is initialised after other NGPC has already sent data, a byte of data
;*        will be lost.  This means that both NGPCs should have had called LinkUp_Initialise
;*        before negotiation is attempted.
;*
;*        Define _FAKELINKUP to "simulate" the library attempting proper machine-to-machine
;*        link-up on a single machine.  Useful for testing link-up games on emulators.  The
;*        controls will control both players at once.
;*
;**************************************************************************************************


;*******************
;* Private defines *
;*******************

COM_BUF_OK            Equ 000h
COM_RECEIVEERRORS     Equ 00Eh
					   
VECT_COMINIT          Equ 010h
VECT_COMSENDSTART     Equ 011h
VECT_COMRECIVESTART   Equ 012h
VECT_COMCREATEDATA    Equ 013h
VECT_COMGETDATA       Equ 014h
VECT_COMONRTS         Equ 015h
VECT_COMOFFRTS        Equ 016h
VECT_COMSENDSTATUS    Equ 017h
VECT_COMRECIVESTATUS  Equ 018h
VECT_COMCREATEBUFDATA Equ 019h
VECT_COMGETBUFDATA    Equ 01Ah


;*************
;* Variables *
;*************

	UserRAM_DW wWaitTime1
	UserRAM_DW wWaitTime2
	UserRAM_DB bWaitBytesLeft
	UserRAM_DB bHostPlayer
	UserRAM_DB bPlayer1Controls
	UserRAM_DB bPlayer2Controls


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - LinkUp_Initialise
;* Parameters  - void
;* Returns     - void
;* Description - Initialises the link-up code.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_Initialise: Macro {NoExpand}

	;/* Ensure link-up is not being faked */
	IfNDef _FAKELINKUP

		;/* Initialise communications port */
		ldb  rw3, VECT_COMINIT
		calr       SYSTEM_CALL

		;/* Allow data reception */
		ldb  rw3, VECT_COMRECIVESTART
		calr              SYSTEM_CALL

	EndIf

	;/* Reset waiting bytes */
	ldw     (wWaitTime1), 0
	ldb (bWaitBytesLeft), 4

	EndM

;****************************
;* End of LinkUp_Initialise *
;****************************


;**************************************************************************************************
;*
;* Macro       - LinkUp_IsPlayer1
;* Parameters  - void
;* Returns     - "true" if the host machine is player 1, "false" otherwise.
;* Description - Tests if the host machine is player 1.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_IsPlayer1: Macro {NoExpand}

	;/* Test host player number for 1 */
	bit 0, (bHostPlayer)

	EndM

;***************************
;* End of LinkUp_IsPlayer1 *
;***************************


;**************************************************************************************************
;*
;* Macro       - LinkUp_IsPlayer2
;* Parameters  - void
;* Returns     - "true" if the host machine is player 2, "false" otherwise.
;* Description - Tests if the host machine is player 2.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_IsPlayer2: Macro {NoExpand}

	;/* Test host player number for 2 */
	bit 1, (bHostPlayer)

	EndM

;***************************
;* End of LinkUp_IsPlayer2 *
;***************************


;**************************************************************************************************
;*
;* Macro       - LinkUp_SyncMemory_NGPCs
;* Parameters  - pAddressHost: The address containing the data from the host machine.
;*               pAddressLink: The address to place the data from the linked machine.
;*               nLength     : The length of the data.
;* Returns     - void
;* Description - Synchronises two buffers between the 2 NGPCs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_SyncMemory_NGPCs: Macro pAddressHost, pAddressLink, nLength, {NoExpand}

	;/* Ensure link-up is not being faked */
	IfNDef _FAKELINKUP

		;/* Ensure a non-zero buffer is being synchronised */
		If nLength > 0

			;/* Create buffer for sending */
			ld   xhl3,          pAddressHost
			ldb   rb3,               nLength
SendAgain:
			ldb   rw3, VECT_COMCREATEBUFDATA
			calr                 SYSTEM_CALL
			cpb   rb3,                     0
			jr     ne,             SendAgain

			;/* Send buffer */
			ldb  rw3, VECT_COMSENDSTART
			calr            SYSTEM_CALL

			;/* Ensure receive buffer is ready */
StatusAgain:
			ldb  rw3, VECT_COMRECIVESTATUS
			calr               SYSTEM_CALL
			ldw   wa,                 rwa3
			andb   w,    COM_RECEIVEERRORS
			jr    nz,          StatusAgain

			;/* Read buffer */
			ld   xhl3,       pAddressLink
			ldb   rb3,            nLength
GetAgain:
			ldb   rw3, VECT_COMGETBUFDATA
			calr              SYSTEM_CALL
			cpb   rb3,                  0
			jr     ne,           GetAgain

		EndIf

	EndIf

	EndM

;**********************************
;* End of LinkUp_SyncMemory_NGPCs *
;**********************************


;**************************************************************************************************
;*
;* Macro       - LinkUp_SyncMemory_Players
;* Parameters  - pAddress1: The address containing the data for player 1.
;*               pAddress2: The address containing the data for player 2.
;*               nLength  : The length of the data.
;* Returns     - void
;* Description - Synchronises player data between the 2 NGPCs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_SyncMemory_Players: Macro pAddress1, pAddress2, nLength, {NoExpand}

	;/* Check if host NGPC is player 1 */
	LinkUp_IsPlayer1
	jr ne, NotPlayer1

	;/* Send player 1 data and receive player 2 data */
	LinkUp_SyncMemory_NGPCs pAddress1, pAddress2, nLength
	jr End

	;/* Send player 2 data and receive player 1 data */
NotPlayer1:
	LinkUp_SyncMemory_NGPCs pAddress2, pAddress1, nLength

End:
	EndM

;************************************
;* End of LinkUp_SyncMemory_Players *
;************************************


;**************************************************************************************************
;*
;* Macro       - LinkUp_Negotiate
;* Parameters  - pSuccess: The address to jump to if link-up was successful.
;* Returns     - void
;* Description - Attempt to establish link-up between two NGPCs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_Negotiate: Macro pSuccess, {NoExpand}

	;/* Check if link-up is not being faked */
	IfNDef _FAKELINKUP

		;/* Check if this is the first call */
		cpw (wWaitTime1),            0
		jr            ne, NotFirstCall

		;/* Create buffer of four 123 bytes for sending */
		ldb    b,                  4
Again:
		ldb  rw3, VECT_COMCREATEDATA
		ldb  rb3,                123
		calr             SYSTEM_CALL
		cpb  ra3,         COM_BUF_OK
		jr    ne,              Again
		djnz   b,              Again

NotFirstCall:
		;/* Send bytes */
		ldb  rw3, VECT_COMSENDSTART
		calr            SYSTEM_CALL

		;/* Increment the amount of time passed */
		incw (wWaitTime1)

		;/* Ensure receive buffer is ready */
		ldb  rw3, VECT_COMRECIVESTATUS
		calr               SYSTEM_CALL
		ldw   wa,                 rwa3
		andb   w,    COM_RECEIVEERRORS
		jp    nz,                 Skip

		;/* Attempt to read buffer */
		ldb  rw3, VECT_COMGETDATA
		calr          SYSTEM_CALL
		cpb  ra3,      COM_BUF_OK
		jr    ne,            Skip

		;/* Check if received byte is a 123 */
		cpb rb3,  123
		jp   ne, Skip

		;/* Decrement number of received 123 bytes */
		decb (bWaitBytesLeft)

		;/* Check if all 123 bytes have been received */
		cpb (bWaitBytesLeft),    0
		jp                ne, Skip

		;/* Synchronise wait times */
		LinkUp_SyncMemory_NGPCs wWaitTime1, wWaitTime2, 2

		;/* Assume host machine is player 1 */
		ldb a, 1

		;/* Check if host machine has been waiting longer */
		ldw bc, (wWaitTime1)
		cpw bc, (wWaitTime2)
Died:
		jr  eq,         Died
		jr ugt,   NotPlayer2

		;/* Host machine is player 2 */
		ldb a, 2

NotPlayer2:
		;/* Store host machine number */
		ldb (bHostPlayer), a

	;/* Link-up is being faked */
	Else

		;/* Store host machine fake number */
		ldb (bHostPlayer), _FAKELINKUP

	EndIf

	;/* Jump to success address */
	jp pSuccess

Skip:
	EndM

;***************************
;* End of LinkUp_Negotiate *
;***************************


;**************************************************************************************************
;*
;* Macro       - LinkUp_SyncControls
;* Parameters  - void
;* Returns     - void
;* Description - Synchronises the controls between the linked NGPCs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

LinkUp_SyncControls: Macro {NoExpand}

	;/* Check if fake link-up is not being used */
	IfNDef _FAKELINKUP

		;/* Copy host controls for both players */
		ldb                  a, (AInput_Register)
		ldb (bPlayer1Controls),                 a
		ldb (bPlayer2Controls),                 a

		;/* Synchronise players' controls between both machines */
		LinkUp_SyncMemory_Players bPlayer1Controls, bPlayer2Controls, 1

	;/* Fake link-up is being used */
	Else

		;/* Check if player 1 is being faked */
		If _FAKELINKUP == 1

			;/* Copy host controls for player 1 */
			ldb                  a, (AInput_Register)
			ldb (bPlayer1Controls),                 a

		;/* Player 1 is not being faked - check if player 2 is being faked */
		ElseIf _FAKELINKUP == 2

			;/* Copy host controls for player 2 */
			ldb                  a, (AInput_Register)
			ldb (bPlayer2Controls),                 a

		;/* Both players are being faked */
		Else

			;/* Copy host controls for both players */
			ldb                  a, (AInput_Register)
			ldb (bPlayer1Controls),                 a
			ldb (bPlayer2Controls),                 a

		EndIf

	EndIf

	EndM

;******************************
;* End of LinkUp_SyncControls *
;******************************


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FLinkUp_GetPlayer1ControlsAddress
;* Parameters  - void
;* Returns     - The address where player 1's controls are stored.
;* Description - Returns the address where player 1's controls are stored.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FLinkUp_GetPlayer1ControlsAddress Equ bPlayer1Controls

;********************************************
;* End of FLinkUp_GetPlayer1ControlsAddress *
;********************************************


;**************************************************************************************************
;*
;* Function    - FLinkUp_GetPlayer2ControlsAddress
;* Parameters  - void
;* Returns     - The address where player 2's controls are stored.
;* Description - Returns the address where player 2's controls are stored.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FLinkUp_GetPlayer2ControlsAddress Equ bPlayer2Controls

;********************************************
;* End of FLinkUp_GetPlayer2ControlsAddress *
;********************************************


	EndIf ;// IfNDef LINKUP_INC

;*********************
;* End of LinkUp.inc *
;*********************
