;**************************************************************************************************
;*
;* Memory.inc: Memory manipulation macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef MEMORY_INC
MEMORY_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Memory_Zero
;* Parameters  - pAddress: A pointer to the start of memory to zero.
;*               nSize   : The size of the memory to zero.
;* Returns     - void
;* Description - Zeros a block of memory.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Memory_Zero: Macro pAddress, nSize, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                  pAddress, "Syntax: Memory_Zero <address>, <size:0-65535>"
	StaticAssert_NotBlank                     nSize, "Syntax: Memory_Zero <address>, <size:0-65535>"
	StaticAssert          FInRange(nSize, 0, 65535), "Memory_Zero: size must be in the range 0-65535"

	;/* Ensure memory to clear is at least 0 bytes */
	If (nSize) > 0

		;/* Check if memory to clear is 1 byte */
		If (nSize) == 1

			;/* Load zero byte into address */
			ldb (pAddress), 0

		;/* Memory to clear is not 1 byte - check if it's 2 bytes */
		ElseIf (nSize) == 2

			;/* Load zero word into address */
			ldw (pAddress), 0

		;/* Memory to clear is not 2 bytes - check if it's 3 bytes */
		ElseIf (nSize) == 3

			;/* Load zero word into address followed by zero byte */
			ld     xde, pAddress
			ldw (xde+),        0
			ldb  (xde),        0

		;/* Memory to clear is not 3 bytes - check if it's 4 bytes */
		ElseIf (nSize) == 4

			;/* Load zero double into address */
			xor        xde, xde
			ld  (pAddress), xde

		;/* Memory to clear is not 4 bytes - check if it's 5 bytes */
		ElseIf (nSize) == 5

			;/* Load start address into xde and xhl */
			ld xde, pAddress
			ld xhl,      xde

			;/* Load zero word into start address and increment address in xde */
			ldw (xde+), 0

			;/* Copy zero word from address in xde to address in xhl and increment both addresses, then again for a byte */
			ldiw (xde+), (xhl+)
			ldib (xde+), (xhl+)

		;/* Memory to clear is not 5 bytes - check if it's 6 bytes */
		ElseIf (nSize) == 6

			;/* Load start address into xde and xhl */
			ld xde, pAddress
			ld xhl,      xde

			;/* Load zero word into start address and increment address in xde */
			ldw (xde+), 0

			;/* Copy 2 zero words from address in xde to address in xhl, incrementing both addresses each time */
			ldiw (xde+), (xhl+)
			ldiw (xde+), (xhl+)

		;/* Memory to clear is not 6 bytes */
		Else

			;/* Load start address into xde and xhl */
			ld xde, pAddress
			ld xhl,      xde

			;/* Load zero word into start address and increment address in xde */
			ldw (xde+), 0

			;/* Store 1 less than the number of words to read in bc */
			ldw bc, Int((nSize) / 2) - 1

			;/* Copy zero word from address in xde to address in xhl, increment both addresses and repeat bc times */
			ldirw (xde+), (xhl+)

			;/* Check if number of words being read is odd */
			If (nSize) <> Int((nSize) / 2) * 2

				;/* Zero extra byte */
				ldib (xde+), (xhl+)

			EndIf

		EndIf

	EndIf

	EndM

;**********************
;* End of Memory_Zero *
;**********************


;**************************************************************************************************
;*
;* Macro       - Memory_Copy
;* Parameters  - pSource     : A pointer to the start of memory to copy from.
;*               pDestination: A pointer to the start of memory to copy to.
;*               nSize       : The size of the memory to copy.
;* Returns     - void
;* Description - Copies a block of memory.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Memory_Copy: Macro pSource, pDestination, nSize, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   pSource, "Syntax: Memory_Copy <source>, <destination>, <size:0-65535>"
	StaticAssert_NotBlank              pDestination, "Syntax: Memory_Copy <source>, <destination>, <size:0-65535>"
	StaticAssert_NotBlank                     pSize, "Syntax: Memory_Copy <source>, <destination>, <size:0-65535>"
	StaticAssert          FInRange(nSize, 0, 65535), "Memory_Copy: size must be in the range 0-65535"

	;/* Ensure memory to copy is at least 0 bytes and pointers are different */
	If ((nSize) > 0) && ((pSource) <> (pDestination))

		;/* Load source address into xhl */
		ld xhl, pSource

		;/* Load destination address into xde */
		ld xde, pDestination

		;/* Store number of words to copy in bc */
		ldw bc, Int((nSize) / 2)

		;/* Copy word from address in xde to address in xhl, increment both addresses and repeat bc times */
		ldirw (xde+), (xhl+)

		;/* Check if number of words being copied is odd */
		If (nSize) <> Int((nSize) / 2) * 2

			;/* Copy extra byte */
			ldib (xde+), (xhl+)

		EndIf

	EndIf

	EndM

;**********************
;* End of Memory_Copy *
;**********************


	EndIf ;// IfNDef MEMORY_INC

;*********************
;* End of Memory.inc *
;*********************
