;**************************************************************************************************
;*
;* Scroll.inc: Scroll plane manipulation macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef SCROLL_INC
SCROLL_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"
	Include "Memory.inc"


;***************************
;* Memory location defines *
;***************************

AScroll1_RAM        Equ 09000h ;// Start of scroll plane 1 RAM
AScroll2_RAM        Equ 09800h ;// Start of scroll plane 2 RAM

AScroll1_PaletteRAM Equ 08280h ;// Start of scroll plane 1 palette RAM
AScroll2_PaletteRAM Equ 08300h ;// Start of scroll plane 2 palette RAM

AScroll1_XOffset    Equ 08032h ;// X offset of scroll plane 1
AScroll1_YOffset    Equ 08033h ;// Y offset of scroll plane 1
AScroll2_XOffset    Equ 08034h ;// X offset of scroll plane 2
AScroll2_YOffset    Equ 08035h ;// Y offset of scroll plane 2

AScroll_Priority    Equ 08030h ;// Scroll plane priority register


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FScroll_SelectValue
;* Parameters  - nPlane : The scroll plane to select (1,2).
;*               nValue1: The value associated with scroll plane 1.
;*               nValue2: The value associated with scroll plane 2.
;* Returns     - The value associated with the specified scroll plane.
;* Description - Returns one of two values depending on a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FScroll_SelectValue Function nPlane, nValue1, nValue2, (nValue1) + (((nValue2) - (nValue1)) * ((nPlane) - 1))

;******************************
;* End of FScroll_SelectValue *
;******************************


;**************************************************************************************************
;*
;* Function    - FScroll_RAMWord
;* Parameters  - nTileIndex   : The index of the tile to use (0-511).
;*               nPaletteIndex: The index of the palette to use (0-15).
;*               bXFlip       : "true" if the tile is flipped horizontally, "false" otherwise.
;*               bYFlip       : "true" if the tile is flipped vertically, "false" otherwise.
;* Returns     - The scroll RAM word representing the supplied parameters.
;* Description - Returns a scroll RAM word.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FScroll_RAMWord Function nTileIndex, nPaletteIndex, bXFlip, bYFlip, ((nTileIndex) & 511) + (((nPaletteIndex) & 15) * 512) + (NonZero(bXFlip) * 32768) + (NonZero(bYFlip) * 16384)

;**************************
;* End of FScroll_RAMWord *
;**************************


;**************************************************************************************************
;*
;* Function    - FScroll_GetXXXAddress
;* Parameters  - nPlane: The scroll plane to return (1,2).
;* Returns     - The address for the specified scroll plane.
;* Description - Returns the correct address for various scroll plane memory locations.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FScroll_GetRAMAddress        Function nPlane, FScroll_SelectValue(nPlane, AScroll1_RAM       , AScroll2_RAM       )
FScroll_GetPaletteRAMAddress Function nPlane, FScroll_SelectValue(nPlane, AScroll1_PaletteRAM, AScroll2_PaletteRAM)
FScroll_GetOffsetXAddress    Function nPlane, FScroll_SelectValue(nPlane, AScroll1_XOffset   , AScroll2_XOffset   )
FScroll_GetOffsetYAddress    Function nPlane, FScroll_SelectValue(nPlane, AScroll1_YOffset   , AScroll2_YOffset   )

;********************************
;* End of FScroll_GetXXXAddress *
;********************************


;**************************************************************************************************
;*
;* Function    - FScroll_GetPaletteEntryAddress
;* Parameters  - nPlane       : The scroll plane to return (1,2).
;*               nPaletteIndex: The index of the palette to return (0-15).
;*               nEntryIndex  : The index of the palette entry to return (1-3).
;* Returns     - The address of the specified palette entry.
;* Description - Returns the address of a scroll plane's palette entry.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FScroll_GetPaletteEntryAddress Function nPlane, nPaletteIndex, nEntryIndex, FScroll_GetPaletteRAMAddress(nPlane) + ((nPaletteIndex) * 8) + ((nEntryIndex) * 2)

;*****************************************
;* End of FScroll_GetPaletteEntryAddress *
;*****************************************


;**************************************************************************************************
;*
;* Function    - FScroll_GetTileAddress
;* Parameters  - nPlane: The scroll plane containing the tile to return (1,2).
;*               nX    : The X-coordinate of the tile to return (0-31).
;*               nY    : The Y-coordinate of the tile to return (0-31).
;* Returns     - The address of the specified tile.
;* Description - Returns the address of a tile on a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FScroll_GetTileAddress Function nPlane, nX, nY, FScroll_GetRAMAddress(nPlane) + (((nX) + ((nY) * 32)) * 2)

;*********************************
;* End of FScroll_GetTileAddress *
;*********************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Scroll_Clear
;* Parameters  - nPlanes       : The scroll plane(s) to clear (1,2,1+2).
;*               nScrollRAMWord: The scroll RAM word to set the tiles to (optional).
;* Returns     - void
;* Description - Clears one or both scroll planes.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_Clear: Macro nPlanes, nScrollRAMWord, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                 nPlanes, "Syntax: Scroll_Clear <planes:1,2,1+2>[, <scroll RAM word>]"
	StaticAssert          FInRange(nPlanes, 1, 3), "Scroll_Clear: planes must be 1, 2 or 1+2"

	;/* Check if both planes are being cleared */
	If nPlanes = 3

		;/* Load address of scroll plane 1 into xde */
		ld xde, AScroll1_RAM

		;/* Load size of 2 planes minus 1 into bc */
		ld bc, (32 * 32 * 2) - 1

	;/* Both planes are not being cleared */
	Else

		;/* Load address of scroll plane into xde */
		ld xde, FScroll_GetRAMAddress(nPlanes)

		;/* Load size of plane minus 1 into bc */
		ld bc, (32 * 32) - 1

	EndIf

	;/* Copy start address into xhl */
	ld xhl, xde

	;/* Check if a scroll RAM word was specified */
	IfNB nScrollRAMWord

		;/* Load scroll RAM word into start address and increment address in xde */
		ldw (xde+), nScrollRAMWord

	;/* No scroll RAM word was specified */
	Else

		;/* Load zero into start address and increment address in xde */
		ldw (xde+), 0

	EndIf

	;/* Copy scroll RAM word from address in xde to address in xhl, increment both addresses and repeat bc times */
	ldirw (xde+), (xhl+)

	EndM

;***********************
;* End of Scroll_Clear *
;***********************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetTile
;* Parameters  - nPlane        : The scroll plane to set the tile on (1,2).
;*               nX            : The X-coordinate of the tile to set (0-31).
;*               nY            : The Y-coordinate of the tile to set (0-31).
;*               nScrollRAMWord: The scroll RAM word to set the tile to.
;* Returns     - void
;* Description - Sets a tile on a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetTile: Macro nPlane, nX, nY, nScrollRAMWord, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                  nPlane, "Syntax: Scroll_SetTile <plane:1,2>, <x:0-31>, <y:0-31>, <scroll RAM word>"
	StaticAssert_NotBlank                      nX, "Syntax: Scroll_SetTile <plane:1,2>, <x:0-31>, <y:0-31>, <scroll RAM word>"
	StaticAssert_NotBlank                      nY, "Syntax: Scroll_SetTile <plane:1,2>, <x:0-31>, <y:0-31>, <scroll RAM word>"
	StaticAssert_NotBlank          nScrollRAMWord, "Syntax: Scroll_SetTile <plane:1,2>, <x:0-31>, <y:0-31>, <scroll RAM word>"
	StaticAssert          FInRange(nPlane, 1,  2), "Scroll_SetTile: plane must be 1 or 2"
	StaticAssert          FInRange(nX    , 0, 31), "Scroll_SetTile: x must be in the range 0-31"
	StaticAssert          FInRange(nY    , 0, 31), "Scroll_SetTile: y must be in the range 0-31"

	;/* Set tile */
	ldw (FScroll_GetTileAddress(nPlane, nX, nY)), nScrollRAMWord

	EndM

;*************************
;* End of Scroll_SetTile *
;*************************


;**************************************************************************************************
;*
;* Macro       - Scroll_InstallPalettes
;* Parameters  - pPalettesStart: The start address of the block containing the palettes to
;*                               install.
;*               pPalettesEnd  : The end address of the block.
;*               nPaletteIndex : The index of the first palette to install to (0-15).
;* Returns     - void
;* Description - Installs a block of palettes to scroll palette RAM.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_InstallPalettes: Macro nPlane, pPalettesStart, pPalettesEnd, nPaletteIndex, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                                                            nPlane, "Syntax: Scroll_InstallPalettes <plane:1,2>, <start address>, <end address>, <palette:0-15>"
	StaticAssert_NotBlank                                                    pPalettesStart, "Syntax: Scroll_InstallPalettes <plane:1,2>, <start address>, <end address>, <palette:0-15>"
	StaticAssert_NotBlank                                                      pPalettesEnd, "Syntax: Scroll_InstallPalettes <plane:1,2>, <start address>, <end address>, <palette:0-15>"
	StaticAssert_NotBlank                                                     nPaletteIndex, "Syntax: Scroll_InstallPalettes <plane:1,2>, <start address>, <end address>, <palette:0-15>"
	StaticAssert                                         (pPalettesStart) <= (pPalettesEnd), "Scroll_InstallPalettes: start address must be less than end address"
	StaticAssert                                             FInRange(nPaletteIndex, 0, 15), "Scroll_InstallPalettes: palette must be in the range 0-15"
	StaticAssert          (nPaletteIndex) + (((pPalettesEnd) - (pPalettesStart)) / 8) <= 16, "Scroll_InstallPalettes: specified parameters will result in overwriting past end of palette RAM"

	;/* Copy palettes from address to tile RAM */
	Memory_Copy pPalettesStart, FScroll_GetPaletteRAMAddress(nPlane) + ((nPaletteIndex) * 8), (pPalettesEnd) - (pPalettesStart)

	EndM

;*********************************
;* End of Scroll_InstallPalettes *
;*********************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetPaletteEntry
;* Parameters  - nPlane       : The scroll plane to set the palette entry for (1,2).
;*               nPaletteIndex: The palette containing the entry to set (0-15).
;*               nEntryIndex  : The index of the colour to set (1-3).
;*               NCOLOUR      : The colour to set.
;* Returns     - void
;* Description - Sets a colour in a scroll plane palette.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetPaletteEntry: Macro nPlane, nPaletteIndex, nEntryIndex, NCOLOUR, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                         nPlane, "Syntax: Scroll_SetPaletteEntry <plane:1,2>, <palette:0-15>, <entry:1-3>, <colour>"
	StaticAssert_NotBlank                  nPaletteIndex, "Syntax: Scroll_SetPaletteEntry <plane:1,2>, <palette:0-15>, <entry:1-3>, <colour>"
	StaticAssert_NotBlank                    nEntryIndex, "Syntax: Scroll_SetPaletteEntry <plane:1,2>, <palette:0-15>, <entry:1-3>, <colour>"
	StaticAssert_NotBlank                        NCOLOUR, "Syntax: Scroll_SetPaletteEntry <plane:1,2>, <palette:0-15>, <entry:1-3>, <colour>"
	StaticAssert          FInRange(nPlane       , 1,  2), "Scroll_SetPaletteEntry: plane must be 1 or 2"
	StaticAssert          FInRange(nPaletteIndex, 0, 15), "Scroll_SetPaletteEntry: palette must be in the range 0-15"
	StaticAssert          FInRange(nEntryIndex  , 1,  3), "Scroll_SetPaletteEntry: entry must be in the range 1-3"

	;/* Check if colour is held in a register */
	If ~~FIs16BitRegister("NCOLOUR")

		StaticAssert FIsAColour(NCOLOUR), "Scroll_SetPaletteEntry: invalid colour"

	EndIf

	;/* Set palette entry */
	ldw (FScroll_GetPaletteEntryAddress(nPlane, nPaletteIndex, nEntryIndex)), NCOLOUR

	EndM

;*********************************
;* End of Scroll_SetPaletteEntry *
;*********************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetPalette
;* Parameters  - nPlane       : The scroll plane to set the palette for (1,2).
;*               nPaletteIndex: The palette to set (0-15).
;*               nColour1     : The colour to set for palette entry 1.
;*               nColour2     : The colour to set for palette entry 2.
;*               nColour3     : The colour to set for palette entry 3.
;* Returns     - void
;* Description - Sets an entire scroll plane palette.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetPalette: Macro nPlane, nPaletteIndex, nColour1, nColour2, nColour3, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                           nPlane, "Syntax: Scroll_SetPalette <plane:1,2>, <palette:0-15>, <colour 1>, <colour 2>, <colour 3>"
	StaticAssert_NotBlank                    nPaletteIndex, "Syntax: Scroll_SetPalette <plane:1,2>, <palette:0-15>, <colour 1>, <colour 2>, <colour 3>"
	StaticAssert_NotBlank                         nColour1, "Syntax: Scroll_SetPalette <plane:1,2>, <palette:0-15>, <colour 1>, <colour 2>, <colour 3>"
	StaticAssert_NotBlank                         nColour2, "Syntax: Scroll_SetPalette <plane:1,2>, <palette:0-15>, <colour 1>, <colour 2>, <colour 3>"
	StaticAssert_NotBlank                         nColour3, "Syntax: Scroll_SetPalette <plane:1,2>, <palette:0-15>, <colour 1>, <colour 2>, <colour 3>"
	StaticAssert          FInRange  (nPlane       , 1,  2), "Scroll_SetPalette: plane must be 1 or 2"
	StaticAssert          FInRange  (nPaletteIndex, 0, 15), "Scroll_SetPalette: palette must be in the range 0-15"
	StaticAssert          FIsAColour(nColour1            ), "Scroll_SetPalette: colour 1 is invalid"
	StaticAssert          FIsAColour(nColour2            ), "Scroll_SetPalette: colour 2 is invalid"
	StaticAssert          FIsAColour(nColour3            ), "Scroll_SetPalette: colour 3 is invalid"

	;/* Set palette entries */
	Scroll_SetPaletteEntry nPlane, nPaletteIndex, 1, nColour1
	Scroll_SetPaletteEntry nPlane, nPaletteIndex, 2, nColour2
	Scroll_SetPaletteEntry nPlane, nPaletteIndex, 3, nColour3

	EndM

;****************************
;* End of Scroll_SetPalette *
;****************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetOffset
;* Parameters  - nPlane: The scroll plane to set the offset for (1,2).
;*               nX    : The X offset to set ((-255)-255).
;*               nY    : The Y offset to set ((-255)-255).
;* Returns     - void
;* Description - Sets the offset of a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetOffset: Macro nPlane, nX, nY, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                      nPlane, "Syntax: Scroll_SetOffset <plane:1,2>, <x:(-255)-255>, <y:(-255)-255>"
	StaticAssert_NotBlank                          nX, "Syntax: Scroll_SetOffset <plane:1,2>, <x:(-255)-255>, <y:(-255)-255>"
	StaticAssert_NotBlank                          nY, "Syntax: Scroll_SetOffset <plane:1,2>, <x:(-255)-255>, <y:(-255)-255>"
	StaticAssert          FInRange(nPlane,    1,   2), "Scroll_SetOffset: plane must be 1 or 2"
	StaticAssert          FInRange(    nX, -255, 255), "Scroll_SetOffset: x must be in the range (-255)-255"
	StaticAssert          FInRange(    nY, -255, 255), "Scroll_SetOffset: y must be in the range (-255)-255"

	;/* Set offset */
	ldw (FScroll_GetOffsetXAddress(nPlane)), ((nX) & 255) + (((nY) & 255) << 8)

	EndM

;***************************
;* End of Scroll_SetOffset *
;***************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetXOffset
;* Parameters  - nPlane: The scroll plane to set the offset for (1,2).
;*               nX    : The X offset to set ((-255)-255).
;* Returns     - void
;* Description - Sets the X offset of a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetXOffset: Macro nPlane, nX, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                      nPlane, "Syntax: Scroll_SetXOffset <plane:1,2>, <x:(-255)-255>"
	StaticAssert_NotBlank                          nX, "Syntax: Scroll_SetXOffset <plane:1,2>, <x:(-255)-255>"
	StaticAssert          FInRange(nPlane,    1,   2), "Scroll_SetXOffset: plane must be 1 or 2"
	StaticAssert          FInRange(    nX, -255, 255), "Scroll_SetXOffset: x must be in the range (-255)-255"

	;/* Set X offset */
	ldb (FScroll_GetOffsetXAddress(nPlane)), ((nX) & 255)

	EndM

;****************************
;* End of Scroll_SetXOffset *
;****************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetYOffset
;* Parameters  - nPlane: The scroll plane to set the offset for (1,2).
;*               nY    : The Y offset to set ((-255)-255).
;* Returns     - void
;* Description - Sets the Y offset of a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetYOffset: Macro nPlane, nY, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                      nPlane, "Syntax: Scroll_SetYOffset <plane:1,2>, <y:(-255)-255>"
	StaticAssert_NotBlank                          nY, "Syntax: Scroll_SetYOffset <plane:1,2>, <y:(-255)-255>"
	StaticAssert          FInRange(nPlane,    1,   2), "Scroll_SetYOffset: plane must be 1 or 2"
	StaticAssert          FInRange(    nY, -255, 255), "Scroll_SetYOffset: y must be in the range (-255)-255"

	;/* Set X offset */
	ldb (FScroll_GetOffsetYAddress(nPlane)), ((nY) & 255)

	EndM

;****************************
;* End of Scroll_SetYOffset *
;****************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetToFront
;* Parameters  - nPlane: The scroll plane to set to the front (1,2).
;* Returns     - void
;* Description - Sets a specified scroll plane to the front.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetToFront: Macro nPlane, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                 nPlane, "Syntax: Scroll_SetToFront <plane:1,2>"
	StaticAssert          FInRange(nPlane, 1, 2), "Scroll_SetToFront: plane must be 1 or 2"

	;/* Set scroll plane priority register */
	ldb (AScroll_Priority), FScroll_SelectValue(nPlane, 0, 128)

	EndM

;****************************
;* End of Scroll_SetToFront *
;****************************


;**************************************************************************************************
;*
;* Macro       - Scroll_SetRectangle
;* Parameters  - nPlane        : The scroll plane to set the rectangle on (1,2).
;*               nX            : The X-coordinate of the top-left of the rectangle (0-31).
;*               nY            : The Y-coordinate of the top-left of the rectangle (0-31).
;*               nWidth        : The width of the rectangle (0-31).
;*               nHeight       : The height of the rectangle (0-31).
;*               nScrollRAMWord: The scroll RAM word of the top-left of the rectangle.
;*               HORZORVERT    : "horz" if the rectangle is being drawn horizontally, "vert" if
;*                               it is being drawn vertically.
;* Returns     - void
;* Description - Sets a rectangle of scroll plane tiles based on a initial top-left scroll RAM
;*               word, incrementing the tile index for each tile.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_SetRectangle: Macro nPlane, nX, nY, nWidth, nHeight, nScrollRAMWord, HORZORVERT;, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                        nPlane, "Syntax: Scroll_SetRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <scroll RAM word>[, <direction:horz|vert>]"
	StaticAssert_NotBlank                            nX, "Syntax: Scroll_SetRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <scroll RAM word>[, <direction:horz|vert>]"
	StaticAssert_NotBlank                            nY, "Syntax: Scroll_SetRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <scroll RAM word>[, <direction:horz|vert>]"
	StaticAssert_NotBlank                        nWidth, "Syntax: Scroll_SetRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <scroll RAM word>[, <direction:horz|vert>]"
	StaticAssert_NotBlank                       nHeight, "Syntax: Scroll_SetRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <scroll RAM word>[, <direction:horz|vert>]"
	StaticAssert_NotBlank                nScrollRAMWord, "Syntax: Scroll_SetRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <scroll RAM word>[, <direction:horz|vert>]"
	StaticAssert          FInRange(      nPlane, 1,  2), "Scroll_SetRectangle: plane must be 1 or 2"
	StaticAssert          FInRange(          nX, 0, 31), "Scroll_SetRectangle: x must be in the range 0-31"
	StaticAssert          FInRange(          nY, 0, 31), "Scroll_SetRectangle: y must be in the range 0-31"
	StaticAssert          FInRange(      nWidth, 0, 31), "Scroll_SetRectangle: width must be in the range 0-31"
	StaticAssert          FInRange(     nHeight, 0, 31), "Scroll_SetRectangle: height must be in the range 0-31"
	StaticAssert          FInRange(nX +  nWidth, 0, 32), "Scroll_SetRectangle: x + width must be in the range 0-32"
	StaticAssert          FInRange(nY + nHeight, 0, 32), "Scroll_SetRectangle: y + height must be in the range 0-32"

	;/* Check if a direction was specified */
	IfNB HORZORVERT

		;/* Validate direction */
		StaticAssert (UpString("HORZORVERT") == "HORZ") || (UpString("HORZORVERT") == "VERT"), "Scroll_SetRectangle: direction must be \horz\I or \vert\I"

Scroll_SetRectangle_bHorizontal Eval UpString("HORZORVERT") == "HORZ"

	;/* A direction was not specified */
	Else

Scroll_SetRectangle_bHorizontal Eval TRUE

	EndIf

	;/* Ensure both width and height are non-zero */
	If ((nWidth) <> 0) && ((nHeight) <> 0)

		;/* Check if rectangle is a single tile */
		If ((nWidth) == 1) && ((nHeight) == 1)

			;/* Set single tile */
			Scroll_SetTile nPlane, nX, nY, nScrollRAMWord

		;/* Rectangle is not a single tile */
		Else

			;/* Store scroll RAM word of top-left tile in wa */
			ldw wa, nScrollRAMWord

			;/* Store address of top-left tile in xde */
			ld xde, FScroll_GetTileAddress(nPlane, nX, nY)

			;/* Check if height is not one */
			If (nHeight) <> 1

				;/* Store height of rectangle in hl */
				ldw hl, nHeight
NextLine:

			EndIf

			;/* Check if width is not one */
			If (nWidth) <> 1

				;/* Store width of rectangle in bc */
				ldw bc, nWidth
NextTile:

			EndIf

			;/* Copy scroll RAM word into scroll plane and move xde to next tile */
			ldw (xde+), wa

			;/* Check if rectangle is being drawn horizontally or is of unit width/height */
			If (Scroll_SetRectangle_bHorizontal) || ((nWidth) == 1) || ((nHeight) == 1)

				;/* Increment scroll RAM word by 1 */
				incw wa

			;/* Rectangle is being drawn vertically and rectangle is not of unit width/height */
			Else

				;/* Increment scroll RAM word by height */
				addw wa, nHeight

			EndIf

			;/* Check if width is not one */
			If (nWidth) <> 1

				;/* Repeat tile copy bc times */
				djnz bc, NextTile

			EndIf

			;/* Check if height is not one */
			If (nHeight) <> 1

				;/* Move scroll RAM address to start of next line */
				add xde, 2 * (32 - (nWidth))

				;/* Check if rectangle is being drawn vertically and width isn't 1 */
				If (~~Scroll_SetRectangle_bHorizontal) && ((nWidth) <> 1)

					;/* Decrement scroll RAM word by one less than height */
					subw wa, ((nWidth) * (nHeight)) - 1

				EndIf

				;/* Repeat line copy hl times */
				djnz hl, NextLine

			EndIf

		EndIf

	EndIf

	EndM

;******************************
;* End of Scroll_SetRectangle *
;******************************


;**************************************************************************************************
;*
;* Macro       - Scroll_CopyRectangle
;* Parameters  - nPlane : The scroll plane to copy the rectangle to (1,2).
;*               nX     : The X-coordinate of the top-left of the rectangle (0-31).
;*               nY     : The Y-coordinate of the top-left of the rectangle (0-31).
;*               nWidth : The width of the rectangle (0-31).
;*               nHeight: The height of the rectangle (0-31).
;*               pSource: The address of the rectangle to copy.
;* Returns     - void
;* Description - Copies a rectangle of scroll RAM tiles to a scroll plane.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Scroll_CopyRectangle: Macro nPlane, nX, nY, nWidth, nHeight, pSource, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                        nPlane, "Syntax: Scroll_CopyRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <address>"
	StaticAssert_NotBlank                            nX, "Syntax: Scroll_CopyRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <address>"
	StaticAssert_NotBlank                            nY, "Syntax: Scroll_CopyRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <address>"
	StaticAssert_NotBlank                        nWidth, "Syntax: Scroll_CopyRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <address>"
	StaticAssert_NotBlank                       nHeight, "Syntax: Scroll_CopyRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <address>"
	StaticAssert_NotBlank                       pSource, "Syntax: Scroll_CopyRectangle <plane:1,2>, <x:0-31>, <y:0-31>, <width:0-31>, <height:0-31>, <address>"
	StaticAssert          FInRange(      nPlane, 1,  2), "Scroll_CopyRectangle: plane must be 1 or 2"
	StaticAssert          FInRange(          nX, 0, 31), "Scroll_CopyRectangle: x must be in the range 0-31"
	StaticAssert          FInRange(          nY, 0, 31), "Scroll_CopyRectangle: y must be in the range 0-31"
	StaticAssert          FInRange(      nWidth, 0, 31), "Scroll_CopyRectangle: width must be in the range 0-31"
	StaticAssert          FInRange(     nHeight, 0, 31), "Scroll_CopyRectangle: height must be in the range 0-31"
	StaticAssert          FInRange(nX +  nWidth, 0, 32), "Scroll_CopyRectangle: x + width must be in the range 0-32"
	StaticAssert          FInRange(nY + nHeight, 0, 32), "Scroll_CopyRectangle: y + height must be in the range 0-32"

	;/* Ensure both width and height are non-zero */
	If ((nWidth) <> 0) && ((nHeight) <> 0)

		;/* Check if rectangle is a single tile */
		If ((nWidth) == 1) && ((nHeight) == 1)

			;/* Load tile into de */
			ldw de, (pSource)

			;/* Set single tile */
			Scroll_SetTile nPlane, nX, nY, de

		;/* Rectangle is not a single tile */
		Else

			;/* Store source address in xhl */
			ld xhl, pSource

			;/* Store address of top-left tile in xde */
			ld xde, FScroll_GetTileAddress(nPlane, nX, nY)

			;/* Check if height is not one */
			If (nHeight) <> 1

				;/* Store height of rectangle in wa */
				ldw wa, nHeight
NextLine:

			EndIf

			;/* Check if width is not one */
			If (nWidth) <> 1

				;/* Store width of rectangle in bc */
				ldw bc, nWidth
NextTile:

			EndIf

			;/* Check if width is not one */
			If (nWidth) <> 1

				;/* Copy scroll RAM word from source into scroll plane and move both xde and xhl to next tile and repeat bc times */
				ldirw (xde+), (xhl+)

			;/* Width is one */
			Else

				;/* Copy scroll RAM word from source into scroll plane and move both xde and xhl to next tile */
				ldiw (xde+), (xhl+)

			EndIf

			;/* Check if height is not one */
			If (nHeight) <> 1

				;/* Move scroll RAM address to start of next line */
				add xde, 2 * (32 - (nWidth))

				;/* Repeat line copy wa times */
				djnz wa, NextLine

			EndIf

		EndIf

	EndIf

	EndM

;*******************************
;* End of Scroll_CopyRectangle *
;*******************************


	EndIf ;// IfNDef SCROLL_INC

;*********************
;* End of Scroll.inc *
;*********************
