;**************************************************************************************************
;*
;* Sprite.inc: Sprite manipulation macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef SPRITE_INC
SPRITE_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"
	Include "Memory.inc"


;***************************
;* Memory location defines *
;***************************

ASprite_PaletteRAM      Equ 08200h ;// Start of sprite palette RAM
ASprite_PaletteIndexRAM Equ 08C00h ;// Start of sprite palette index RAM
ASprite_RAM             Equ 08800h ;// Start of sprite RAM

ASprite_XOffset         Equ 08020h ;// X offset of all sprites
ASprite_YOffset         Equ 08021h ;// Y offset of all sprites


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FSprite_GetDataAddress
;* Parameters  - nSprite: The sprite to return (0-63).
;* Returns     - The address of the specified sprite's data.
;* Description - Returns the address of a sprite's data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FSprite_GetDataAddress Function nSprite, ASprite_RAM + ((nSprite) * 4)

;*********************************
;* End of FSprite_GetDataAddress *
;*********************************


;**************************************************************************************************
;*
;* Function    - FSprite_GetPositionAddress
;* Parameters  - nSprite: The sprite to return (0-63).
;* Returns     - The address of the specified sprite's position.
;* Description - Returns the address of a sprite's position.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FSprite_GetPositionAddress  Function nSprite, FSprite_GetDataAddress(nSprite) + 2
FSprite_GetPositionXAddress Function nSprite, FSprite_GetDataAddress(nSprite) + 2
FSprite_GetPositionYAddress Function nSprite, FSprite_GetDataAddress(nSprite) + 3

;*************************************
;* End of FSprite_GetPositionAddress *
;*************************************


;**************************************************************************************************
;*
;* Function    - FSprite_GetPaletteEntryAddress
;* Parameters  - nPaletteIndex: The index of the palette to return (0-15).
;*               nEntryIndex  : The index of the palette entry to return (1-3).
;* Returns     - The address of the specified palette entry.
;* Description - Returns the address of a sprite palette entry.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FSprite_GetPaletteEntryAddress Function nPaletteIndex, nEntryIndex, ASprite_PaletteRAM + ((nPaletteIndex) * 8) + ((nEntryIndex) * 2)

;*****************************************
;* End of FSprite_GetPaletteEntryAddress *
;*****************************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Sprite_InstallPalettes
;* Parameters  - pPalettesStart: The start address of the block containing the palettes to
;*                               install.
;*               pPalettesEnd  : The end address of the block.
;*               nPaletteIndex : The index of the first palette to install to (0-15).
;* Returns     - void
;* Description - Installs a block of palettes to sprite palette RAM.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Sprite_InstallPalettes: Macro pPalettesStart, pPalettesEnd, nPaletteIndex, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                                                    pPalettesStart, "Syntax: Sprite_InstallPalettes <start address>, <end address>, <palette:0-15>"
	StaticAssert_NotBlank                                                      pPalettesEnd, "Syntax: Sprite_InstallPalettes <start address>, <end address>, <palette:0-15>"
	StaticAssert_NotBlank                                                     nPaletteIndex, "Syntax: Sprite_InstallPalettes <start address>, <end address>, <palette:0-15>"
	StaticAssert                                         (pPalettesStart) <= (pPalettesEnd), "Sprite_InstallPalettes: start address must be less than end address"
	StaticAssert                                             FInRange(nPaletteIndex, 0, 15), "Sprite_InstallPalettes: index must be in the range 0-15"
	StaticAssert          (nPaletteIndex) + (((pPalettesEnd) - (pPalettesStart)) / 8) <= 16, "Sprite_InstallPalettes: specified parameters will result in overwriting past end of palette RAM"

	;/* Copy palettes from address to sprite palette RAM */
	Memory_Copy pPalettesStart, ASprite_PaletteRAM + ((nPaletteIndex) * 8), (pPalettesEnd) - (pPalettesStart)

	EndM

;*********************************
;* End of Scroll_InstallPalettes *
;*********************************


Sprite_SetPosition: Macro nSprite, nX, nY, {NoExpand}

	ldw (FSprite_GetPositionAddress(nSprite)), ((nX) & 255) + (((nY) & 255) << 8)

	EndM


Sprite_SetXPosition: Macro sprite, nX, {NoExpand}

	ldb (FSprite_GetPositionXAddress(sprite)), nX

	EndM


Sprite_SetYPosition: Macro nSprite, nY, {NoExpand}

	ldb (FSprite_GetPositionYAddress(nSprite)), nY

	EndM


Sprite_SetChain: Macro nSprite, bX, bY, {NoExpand}

	andb (FSprite_GetDataAddress(nSprite) + 1), 11111001b
	orb  (FSprite_GetDataAddress(nSprite) + 1), (NonZero(bX) << 2) + (NonZero(bY) << 1)

	EndM


Sprite_SetFlip: Macro nSprite, bX, bY, {NoExpand}

	andb (FSprite_GetDataAddress(nSprite) + 1), 00111111b
	orb  (FSprite_GetDataAddress(nSprite) + 1), (NonZero(bX) << 7) + (NonZero(bY) << 6)

	EndM

Sprite_Enable: Macro nSprite, bScroll1, bScroll2, {NoExpand}

	andb (FSprite_GetDataAddress(nSprite) + 1), 11100111b
	orb  (FSprite_GetDataAddress(nSprite) + 1), (NonZero(bScroll1) << 4) + (NonZero(bScroll2) << 3)

	EndM


Sprite_SetPaletteIndex: Macro sprite, index, {NoExpand}

	ldb (ASprite_PaletteIndexRAM + (sprite)), (index) + 0

	EndM

Sprite_SetTileIndex: Macro nSprite, nIndex, {NoExpand}

	If nIndex > 255

		set 0, (FSprite_GetDataAddress(nSprite) + 1)
		ldb (FSprite_GetDataAddress(nSprite)), (nIndex) & 255

	Else

		res 0, (FSprite_GetDataAddress(nSprite) + 1)
		ldb (FSprite_GetDataAddress(nSprite)), nIndex

	EndIf

	EndM


Sprite_JoinRectangle: Macro first, width, height, {NoExpand}

yspr eval 0
	while yspr < (height)
xspr eval 0
		while xspr < (width)
			if (xspr <> 0)
				Sprite_SetPosition (first) + (yspr * (width)) + xspr,                         8, 0
			elseif (yspr <> 0)
				Sprite_SetPosition (first) + (yspr * (width)) + xspr, 256 - (((width) - 1) * 8), 8
			endif
xspr eval xspr + 1

		endm
yspr eval yspr + 1

	endm
spr eval 1
	while spr < ((height) * (width))
		Sprite_SetChain (first) + spr, 1, 1
spr eval spr + 1

	endm
	Sprite_SetChain (first), 0, 0

	EndM


	EndIf ;// IfNDef SPRITE_INC

;*********************
;* End of Sprite.inc *
;*********************
