;**************************************************************************************************
;*
;* Tile.inc: Tile RAM manipulation macros for the NGPC.
;*
;* Author  : Steve Robb
;*
;**************************************************************************************************

	IfNDef TILE_INC
TILE_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"
	Include "Memory.inc"


;***************************
;* Memory location defines *
;***************************

ATile_RAM Equ 0A000h ;// Start of tile RAM


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FTile_GetTileAddress
;* Parameters  - nIndex: The index of the tile to return (0-511).
;* Returns     - The address of the specified tile.
;* Description - Returns the address of a tile.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FTile_GetTileAddress Function nIndex, ATile_RAM + ((nIndex) * 16)

;*******************************
;* End of FTile_GetTileAddress *
;*******************************


;**************************************************************************************************
;*
;* Function    - FTile_GetTileIndex
;* Parameters  - pTile: The address of the tile 
;*               pBase: The address of the tile's base.
;* Returns     - The index of the tile relative to its base.
;* Description - Returns the index of a tile relative to its base.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FTile_GetTileIndex Function pTile, pBase, ((pTile) - (pBase)) / 16

;*****************************
;* End of FTile_GetTileIndex *
;*****************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Tile_Blank
;* Parameters  - void
;* Returns     - void
;* Description - Defines a blank tile.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Tile_Blank: Macro {NoExpand}

	;/* Write 8 zero words */
	DW 0, 0, 0, 0, 0, 0, 0, 0

	EndM

;*********************
;* End of Tile_Blank *
;*********************


;**************************************************************************************************
;*
;* Macro       - Tile_Install
;* Parameters  - pTilesStart: The start address of the block containing the tiles to install.
;*               pTilesEnd  : The end address of the block.
;*               nIndex     : The index of the first tile to install to.
;* Returns     - void
;* Description - Installs a block of tiles to tile RAM.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Tile_Install: Macro pTilesStart, pTilesEnd, nIndex, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                                                  pTilesStart, "Syntax: Tile_Install <start address>, <end address>, <index:0-511>"
	StaticAssert_NotBlank                                                    pTilesEnd, "Syntax: Tile_Install <start address>, <end address>, <index:0-511>"
	StaticAssert_NotBlank                                                       nIndex, "Syntax: Tile_Install <start address>, <end address>, <index:0-511>"
	StaticAssert                                          (pTilesStart) <= (pTilesEnd), "Tile_Install: start address must be less than end address"
	StaticAssert                                              FInRange(nIndex, 0, 511), "Tile_Install: index must be in the range 0-511"
	StaticAssert          (nIndex) + FTile_GetTileIndex(pTilesEnd, pTilesStart) <= 512, "Tile_Install: specified parameters will result in overwriting past end of tile RAM"

	;/* Copy tiles from address to tile RAM */
	Memory_Copy pTilesStart, FTile_GetTileAddress(nIndex), (pTilesEnd) - (pTilesStart)

	EndM

;***********************
;* End of Tile_Install *
;***********************


	EndIf ;// IfNDef TILE_INC

;*******************
;* End of Tile.inc *
;*******************
