;**************************************************************************************************
;*
;* Window.inc: Window manipulation macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef WINDOW_INC
WINDOW_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "2D.inc"
	Include "Assert.inc"


;***************************
;* Memory location defines *
;***************************

AWindow_XPosition Equ 08002h ;// X position of window
AWindow_YPosition Equ 08003h ;// Y position of window
AWindow_Width     Equ 08004h ;// Width of window
AWindow_Height    Equ 08005h ;// Height of window
AWindow_Palettes  Equ 083F0h ;// Start of window colour palettes


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FWindow_GetPaletteEntryAddress
;* Parameters  - nIndex: The index of the palette entry to return (0-7).
;* Returns     - The address of the specified window palette entry.
;* Description - Returns the address of a window palette entry.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FWindow_GetPaletteEntryAddress Function nIndex, AWindow_Palettes + ((nIndex) * 2)

;*****************************************
;* End of FWindow_GetPaletteEntryAddress *
;*****************************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Window_SetPosition
;* Parameters  - nX: The X-coordinate of the window.
;*               nY: The Y-coordinate of the window.
;* Returns     - void
;* Description - Sets the position of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetPosition: Macro nX, nY, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nX, "Syntax: Window_SetPosition <x:0-159>, <y:0-151>"
	StaticAssert_NotBlank                   nY, "Syntax: Window_SetPosition <x:0-159>, <y:0-151>"
	StaticAssert          FInRange(nX, 0, 159), "Window_SetPosition: x must be in the range 0-159"
	StaticAssert          FInRange(nY, 0, 151), "Window_SetPosition: y must be in the range 0-151"

	;/* Set window position */
	ldw (AWindow_XPosition), (nX) | ((nY) << 8)

	EndM

;*****************************
;* End of Window_SetPosition *
;*****************************


;**************************************************************************************************
;*
;* Macro       - Window_SetXPosition
;* Parameters  - nX: The X-coordinate of the window.
;* Returns     - void
;* Description - Sets the X-position of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetXPosition: Macro nX, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nX, "Syntax: Window_SetXPosition <x:0-159>"
	StaticAssert          FInRange(nX, 0, 159), "Window_SetXPosition: x must be in the range 0-159"

	;/* Set window X position */
	ldb (AWindow_XPosition), nX

	EndM

;******************************
;* End of Window_SetXPosition *
;******************************


;**************************************************************************************************
;*
;* Macro       - Window_SetYPosition
;* Parameters  - nY: The Y-coordinate of the window.
;* Returns     - void
;* Description - Sets the Y-position of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetYPosition: Macro nY, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nY, "Syntax: Window_SetYPosition <y:0-151>"
	StaticAssert          FInRange(nY, 0, 151), "Window_SetYPosition: y must be in the range 0-151"

	;/* Set window Y position */
	ldb (AWindow_YPosition), nY

	EndM

;******************************
;* End of Window_SetYPosition *
;******************************


;**************************************************************************************************
;*
;* Macro       - Window_SetSize
;* Parameters  - nWidth : The width of the window.
;*               nHeight: The height of the window.
;* Returns     - void
;* Description - Sets the size of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetSize: Macro nWidth, nHeight, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                    nWidth, "Syntax: Window_SetSize <width:0-160>, <height:0-152>"
	StaticAssert_NotBlank                   nHeight, "Syntax: Window_SetSize <width:0-160>, <height:0-152>"
	StaticAssert          FInRange( nWidth, 0, 160), "Window_SetSize: width must be in the range 0-160"
	StaticAssert          FInRange(nHeight, 0, 152), "Window_SetSize: height must be in the range 0-152"

	;/* Set window size */
	ldw (AWindow_Width), (nWidth) | ((nHeight) << 8)

	EndM

;*************************
;* End of Window_SetSize *
;*************************


;**************************************************************************************************
;*
;* Macro       - Window_SetWidth
;* Parameters  - nWidth: The width of the window.
;* Returns     - void
;* Description - Sets the width of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetWidth: Macro nWidth, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nWidth, "Syntax: Window_SetWidth <width:0-160>"
	StaticAssert          FInRange(nWidth, 0, 160), "Window_SetWidth: width must be in the range 0-160"

	;/* Set window width */
	ldb (AWindow_Width), nWidth

	EndM

;**************************
;* End of Window_SetWidth *
;**************************


;**************************************************************************************************
;*
;* Macro       - Window_SetHeight
;* Parameters  - nHeight: The height of the window.
;* Returns     - void
;* Description - Sets the height of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetHeight: Macro nHeight, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nHeight, "Syntax: Window_SetHeight <height:0-152>"
	StaticAssert          FInRange(nHeight, 0, 152), "Window_SetHeight: height must be in the range 0-152"

	;/* Set window height */
	ldb (AWindow_Height), nHeight

	EndM

;***************************
;* End of Window_SetHeight *
;***************************


;**************************************************************************************************
;*
;* Macro       - Window_Set
;* Parameters  - nX: The X-coordinate of the window.
;*               nY: The Y-coordinate of the window.
;*               nWidth : The width of the window.
;*               nHeight: The height of the window.
;* Returns     - void
;* Description - Sets the position and size of the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_Set: Macro nX, nY, nWidth, nHeight, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                             nX, "Syntax: Window_Set <x:0-159>, <y:0-151>, <width:0-160>, <height:0-152>"
	StaticAssert_NotBlank                             nY, "Syntax: Window_Set <x:0-159>, <y:0-151>, <width:0-160>, <height:0-152>"
	StaticAssert_NotBlank                         nWidth, "Syntax: Window_Set <x:0-159>, <y:0-151>, <width:0-160>, <height:0-152>"
	StaticAssert_NotBlank                        nHeight, "Syntax: Window_Set <x:0-159>, <y:0-151>, <width:0-160>, <height:0-152>"
	StaticAssert          FInRange(          nX, 0, 159), "Window_Set: x must be in the range 0-159"
	StaticAssert          FInRange(          nY, 0, 151), "Window_Set: y must be in the range 0-151"
	StaticAssert          FInRange(      nWidth, 0, 160), "Window_Set: width must be in the range 0-160"
	StaticAssert          FInRange(     nHeight, 0, 152), "Window_Set: height must be in the range 0-152"
	StaticAssert          FInRange(nX +  nWidth, 0, 160), "Window_Set: x + width must be in the range 0-160"
	StaticAssert          FInRange(nY + nHeight, 0, 152), "Window_Set: y + height must be in the range 0-152"

	;/* Set window position and size */
	Window_SetPosition     nX,      nY
	Window_SetSize     nWidth, nHeight

	EndM

;*********************
;* End of Window_Set *
;*********************


;**************************************************************************************************
;*
;* Macro       - Window_SetPaletteEntry
;* Parameters  - nIndex : The index of the palette entry to set.
;*               nColour: The colour to set the palette entry to.
;* Returns     - void
;* Description - Sets the colour of a window palette entry.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetPaletteEntry: Macro nIndex, nColour, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nIndex, "Syntax: Window_SetPaletteEntry <index:0-7>, <colour>"
	StaticAssert_NotBlank                  nColour, "Syntax: Window_SetPaletteEntry <index:0-7>, <colour>"
	StaticAssert          FInRange  (nIndex, 0, 7), "Window_SetPaletteEntry: index must be in the range 0-7"
	StaticAssert          FIsAColour(nColour     ), "Window_SetPaletteEntry: colour is invalid"

	;/* Set palette entry */
	ldw (FWindow_GetPaletteEntryAddress(nIndex)), nColour

	EndM

;*********************************
;* End of Window_SetPaletteEntry *
;*********************************


;**************************************************************************************************
;*
;* Macro       - Window_SetPaletteIndex
;* Parameters  - nIndex: The index of the palette entry to use.
;* Returns     - void
;* Description - Sets the index of the palette to be used for the window.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Window_SetPaletteIndex: Macro nIndex, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                 nIndex, "Syntax: Window_SetPaletteIndex <index:0-7>"
	StaticAssert          FInRange(nIndex, 0, 7), "Window_SetPaletteIndex: index must be in the range 0-7"

	;/* Check if any bits of index are not set */
	If nIndex <> 7

		;/* Clear bottom 3 bits of 2D control register */
		andb (A2D_Control), 11111000b
		
	EndIf

	;/* Write index to bottom 3 bits of 2D control register */
	orb (A2D_Control), nIndex

	EndM

;*********************************
;* End of Window_SetPaletteIndex *
;*********************************


	EndIf ;// IfNDef WINDOW_INC

;*********************
;* End of Window.inc *
;*********************
