#include "ngpc.h"
#include "library.h"


volatile u8 VBCounter;

u8 RandomNumberCounter;

// SFX related variables and registers
u8 SFXInstalled;
#define SFXPulse       (*(u8 *)0x7010)
#define SFXPlayBuffer  (*(u8 *)0x7011)
#define SFXLoadPreset  (*(volatile u8 *)0x7012)
#define SFXPlayPreset  (*(u8 *)0x7013)
#define SFXStopAll     (*(u8 *)0x7014)
#define SFXBuffer      (u8 *)0x7020



// dummy Interrupt function used for interrupt that are currently
// unused. To add an interrupt simply supply your own routine in the
// InitNGPC routine in the interrupt vector table initialisation
void __interrupt DummyFunction(void) {}


//////////////////////////////////////////////////////////////////////////////
// VBInterrupt
// Handles the default work required by the VBInterrupt (watchdog,
// shutdown and a standard counter)
//////////////////////////////////////////////////////////////////////////////
void SysShutdown(); // forward reference
void __interrupt VBInterrupt(void)
{
   // clear the watchdog
   WATCHDOG = WATCHDOG_CLEAR;

   // check for power down
   if (USR_SHUTDOWN)
   {
	  SysShutdown();
	  while (1);
   }
   
   // increment a counter
   VBCounter++;

   if (SFXInstalled)
	  SFXPulse = 1;
   
   // TODO: add any other VBI code here.
}

//////////////////////////////////////////////////////////////////////////////
// InitNGPC
// This should be the first function called in main() it initialises the
// NGPC hardware (interrupt vector tables etc)
//////////////////////////////////////////////////////////////////////////////
void InitNGPC(void)
{
   // Set NGP or NGPC mode
   if (OS_VERSION == 0)
   {
	  (*(u8*)0x6f83) &= ~8; // res 3
	  (*(u8*)0x6da0) = 0;
   }

   // set user answer
   USR_ANSWER |= 64; // set 6

   // User hasn't called InstallSoundDriver yet
   SFXInstalled = 0;
   
   // install user interrupt vectors
   SWI3_INT = DummyFunction;
   SWI4_INT = DummyFunction;
   SWI5_INT = DummyFunction;
   SWI6_INT = DummyFunction;
   RTCI_INT = DummyFunction;
   VBL_INT = VBInterrupt;
   Z80_INT = DummyFunction;
   HBL_INT = DummyFunction;
   TI0_INT = DummyFunction;
   TI1_INT = DummyFunction;
   TI2_INT = DummyFunction;
   TI3_INT = DummyFunction;
   STX_INT = DummyFunction;
   SRX_INT = DummyFunction;
   DMA0_INT = DummyFunction;
   DMA1_INT = DummyFunction;
   DMA2_INT = DummyFunction;
   DMA3_INT = DummyFunction;

   ENABLE_INTERRUPTS;
   
   // set screen size
   WIN_X = 0;
   WIN_Y = 0;
   WIN_W = 160;
   WIN_H = 152;

}


//////////////////////////////////////////////////////////////////////////////
// SysShutdown                                                              //
//////////////////////////////////////////////////////////////////////////////
void SysShutdown()
{
   __asm(" ld rw3,0"); // 0 = VECT_SHUTDOWN

   // do the system call
   __asm(" ldf 3");
   __asm(" add w,w");
   __asm(" add w,w");
   __asm(" ld xix,0xfffe00");
   __asm(" ld xix,(xix+w)");
   __asm(" call xix");
}


//////////////////////////////////////////////////////////////////////////////
// SysSetSystemFont                                                         //
//////////////////////////////////////////////////////////////////////////////
void SysSetSystemFont()
{
   __asm(" ld ra3,3"); 
   __asm(" ld rw3,5"); // VECT_SYSFONTSET

   // do the system call
   __asm(" ldf 3");
   __asm(" add w,w");
   __asm(" add w,w");
   __asm(" ld xix, 0xfffe00");
   __asm(" ld xix,(xix+w)");
   __asm(" call xix");
}

//////////////////////////////////////////////////////////////////////////////
// ClearScreen
// Inputs:
//		Plane - Scroll Plane to clear SCR_1_PLANE or SCR_2_PLANE
//////////////////////////////////////////////////////////////////////////////
void ClearScreen(u8 Plane)
{
   int i;
   u16 * Screen;

   switch(Plane)
   {
	  case SCR_1_PLANE:
		 Screen = SCROLL_PLANE_1;
		 break;
	  case SCR_2_PLANE:
		 Screen = SCROLL_PLANE_2;
		 break;
	  default:
		 return;
   }

   for (i = 0; i < 32*32; i ++)
	  Screen[i] = 0;
}

//////////////////////////////////////////////////////////////////////////////
// SetPalette
// Inputs:
//		Plane - Scroll Plane to clear SCR_1_PLANE or SCR_2_PLANE
//		PalleteNo - 0-15 the palette no to set
//      Col0-Col3 - The RGB values for the 4 colours within the palette
//////////////////////////////////////////////////////////////////////////////
void SetPalette(u8 Plane, u8 PaletteNo, u16 Col0, u16 Col1, u16 Col2, u16 Col3)
{
   u16 * thePalette;
   u8 Offset = PaletteNo * 4;

   switch (Plane)
   {
	  case SCR_1_PLANE:
		 thePalette = SCROLL_1_PALETTE;
		 break;
	  case SCR_2_PLANE:
		 thePalette = SCROLL_2_PALETTE;
		 break;
      case SPRITE_PLANE:
         thePalette = SPRITE_PALETTE;
         break;
	  default:
		 return;
   }
   
   thePalette[Offset] = Col0;
   thePalette[Offset+1] = Col1;
   thePalette[Offset+2] = Col2;
   thePalette[Offset+3] = Col3;
}


//////////////////////////////////////////////////////////////////////////////
// PutTile
// Inputs:
//		Plane - Scroll Plane to clear SCR_1_PLANE or SCR_2_PLANE
//		PalleteNo - 0-15 the palette number to set
//      XPos - X Position (0 to 19)
//      YPos - Y Position (0 to 18)
//      TileNo - Tile Number (0 to 511)
//////////////////////////////////////////////////////////////////////////////
void PutTile(u8 Plane, u8 Palette, u8 XPos, u8 YPos, u16 TileNo)
{
   u16 * ScreenPlane1 = SCROLL_PLANE_1;
   u16 * ScreenPlane2 = SCROLL_PLANE_2;

   u16 Offset = ((u16)YPos * 32) + XPos;
   u16 Value = TileNo + ((u16)Palette << 9);

   switch(Plane)
   {
	  case SCR_1_PLANE:
		 ScreenPlane1[Offset] = Value;
		 break;
	  case SCR_2_PLANE:
		 ScreenPlane2[Offset] = Value;
		 break;
	  default:
		 break;
   }
}


//////////////////////////////////////////////////////////////////////////////
// GetTile
// Inputs:
//		Plane - Scroll Plane to clear SCR_1_PLANE or SCR_2_PLANE
//      XPos - X Position (0 to 19)
//      YPos - Y Position (0 to 18)
// Outputs:
//		PalleteNo - 0-15 the palette number to set
//      TileNo - Tile Number (0 to 511)
//////////////////////////////////////////////////////////////////////////////
void GetTile(u8 Plane, u8 *Palette, u8 XPos, u8 YPos, u16 *TileNo)
{
   u16 * ScreenPlane1 = SCROLL_PLANE_1;
   u16 * ScreenPlane2 = SCROLL_PLANE_2;

   u16 Offset = ((u16)YPos * 32) + XPos;
  
   switch(Plane)
   {
      case SCR_1_PLANE:
	 *Palette = (u8)(ScreenPlane1[Offset] >> 9);
	 *TileNo = (ScreenPlane1[Offset] & 511);
	 break;
      case SCR_2_PLANE:
	 *Palette = (u8)(ScreenPlane2[Offset] >> 9);
	 *TileNo = (ScreenPlane2[Offset] & 511);
	 break;
      default:
	 break;
   }
}


//////////////////////////////////////////////////////////////////////////////
// Sleep
// Waits for specified number of VBlanks (60ths of a second)
//////////////////////////////////////////////////////////////////////////////
void Sleep(u8 VBlanks)
{
   VBCounter = 0;
   while (VBCounter < VBlanks)
      ;
}


//////////////////////////////////////////////////////////////////////////////
// PrintDecimal
// Displays a decimal number of the screen padded with leading zeros
// Inputs:
//		Plane - Scroll Plane to clear SCR_1_PLANE or SCR_2_PLANE
//		PalleteNo - 0-15 the palette number to set
//      x - X Position (0 to 19)
//      y - Y Position (0 to 18)
//      Value - The number to display
//      Len - The number of tiles to fill
//            e.g. Value = 15, Len = 4 will display 0015
//////////////////////////////////////////////////////////////////////////////
void PrintDecimal(u8 Plane, u8 PaletteNo, u8 x, u8 y, u16 Value, u8 Len)
{
   u8 i;

   for (i = Len; i > 0; i--)
   {
	  PutTile(Plane, PaletteNo, x+i-1, y, '0' + (Value % 10));
	  Value /= 10;
   }
}

void PrintHex(u8 Plane, u8 PaletteNo, u8 x, u8 y, u16 Value, u8 Len)
{
// Similar to PrintDecimal, but prints in Hexadecimal
// works well with the BCD values returned by GetTime

   u8 i;

   for (i = Len; i > 0; i--)
   {
      if ((Value & 15) < 10)
         PutTile(Plane, PaletteNo, x+i-1, y, '0' + (Value & 15));
      else
         PutTile(Plane, PaletteNo, x+i-1, y, '7' + (Value & 15));

      Value = Value >> 4;
   }
}



//////////////////////////////////////////////////////////////////////////////
// InstallTileSet
// Copies your tile data to the NGPC tileram. Takes a 2D array and a
// length. The length field will equal "sizeof(Tiles)/2" which is the
// number of words in the 2D array.
// This style array can be exported directly from Steve Robb's NeoTile
// NGPC Tile Editing tool
//////////////////////////////////////////////////////////////////////////////
void InstallTileSet(const unsigned short Tiles[][8], u16 Len)
{
   u16 i;
   u16 * TileRam = TILE_RAM;
   u16 * theTiles = (u16 *)Tiles;

   for (i = 0; i < Len; i ++)
   {
	  TileRam[i] = *theTiles++;
   }
}

// similar to InstallTiles but allows you to
// adds tiles to a non-zero location in tile RAM
// useful if a font is installed in the lower tiles
void InstallTileSetAt(const u16 Tiles[][8], u16 Len, u16 Offset)
{
   u16 i;
   u16 * TileRam = TILE_RAM + (Offset * 8);
   u16 * theTiles = (u16 *)Tiles;

   for (i = 0; i < Len; i ++)
   {
      TileRam[i] = *theTiles++;
   }
}



//////////////////////////////////////////////////////////////////////////////
// SetBackgroundColour
// Changes the background colour of NGPC screen.
// The macro RGB() can be used to specify Col.
// e.g SetBackgroundColour(RGB(15,0,0)); will set the background red
//////////////////////////////////////////////////////////////////////////////
void SetBackgroundColour(u16 Col)
{
   BG_PAL = Col;
   BG_COL = 0x80;
}


//////////////////////////////////////////////////////////////////////////////
// PrintString
// Displays a string on the screen at the specified location
// Inputs:
//		Plane - Scroll Plane to clear SCR_1_PLANE or SCR_2_PLANE
//		PalleteNo - 0-15 the palette number to set
//      XPos - X Position (0 to 19)
//      YPos - Y Position (0 to 18)
//      theString - The string to be displayed
//////////////////////////////////////////////////////////////////////////////
void PrintString(u8 Plane, u8 Palette, u8 XPos, u8 YPos, const char * theString)
{
   u16 * Screen;

   switch (Plane)
   {
	  case SCR_1_PLANE:
		 Screen = SCROLL_PLANE_1;
		 break;

	  case SCR_2_PLANE:
		 Screen = SCROLL_PLANE_2;
		 break;

	  default:
		 return;
   }

   while (*theString)
   {
	  u16 Offset = ((u16)YPos * 32) + XPos;
	  u16 Value = *theString + ((u16)Palette << 9);
	  Screen[Offset] = Value;

	  theString++;
	  XPos++;
   }
}



//////////////////////////////////////////////////////////////////////////////
// SetSprite
// Initialise a sprite by mapping a tile number and sprite palette no
// to a sprite number and allowing chaining to be set up.
// Inputs:
//		SpriteNo - 0-63 the sprite to move
//      XPos - X Position (0 to 255)
//      YPos - Y Position (0 to 255)
//////////////////////////////////////////////////////////////////////////////
void SetSprite(u8 SpriteNo, u16 TileNo, u8 Chain, u8 XPos, u8 YPos, u8 PaletteNo)
{
   u16 SprCtrlReg;
   u8 * theSprite = SPRITE_RAM;
   u8 * theSpriteCol = SPRITE_COLOUR;
   
   theSprite += (SpriteNo * 4);
   theSpriteCol += SpriteNo;

   SprCtrlReg = 24; // topmost priority
   if (Chain)
      SprCtrlReg += 6; // v and h chaining

   *(theSprite)   = TileNo;
   *(theSprite+1) = SprCtrlReg+(TileNo>>8);
   *(theSprite+2) = XPos;
   *(theSprite+3) = YPos;

   *theSpriteCol = PaletteNo;
}


//////////////////////////////////////////////////////////////////////////////
// SetSpritePosition
// Moves a already initialise sprite
// Inputs:
//		SpriteNo - 0-63 the sprite to move
//      XPos - X Position (0 to 255)
//      YPos - Y Position (0 to 255)
//////////////////////////////////////////////////////////////////////////////
void SetSpritePosition(u8 SpriteNo, u8 XPos, u8 YPos)
{
   u8 * theSprite = SPRITE_RAM;
   theSprite += (SpriteNo * 4);

   *(theSprite+2) = XPos;
   *(theSprite+3) = YPos;
}


//////////////////////////////////////////////////////////////////////////////
// SFX Sound Engine integration
//////////////////////////////////////////////////////////////////////////////

// the pre-assembled Z80 sound effects driver
const unsigned char SFXDriver_Z80[] =
{
   0x31,0xf0,0x0f,0xc3,0x33,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
   0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,
   0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
   0x00,0x00,0x00,0x3e,0x00,0xfd,0x21,0x50,0x05,0xfd,0x77,0x16,
   0xfd,0x21,0x90,0x05,0xfd,0x77,0x16,0xfd,0x21,0xd0,0x05,0xfd,
   0x77,0x16,0xfd,0x21,0x10,0x06,0xfd,0x77,0x16,0x3a,0x10,0x00,
   0xfe,0x00,0x20,0x22,0x3a,0x11,0x00,0xfe,0x00,0xc2,0x96,0x00,
   0x3a,0x12,0x00,0xfe,0x00,0xc2,0xcc,0x00,0x3a,0x13,0x00,0xfe,
   0x00,0xc2,0x06,0x01,0x3a,0x14,0x00,0xfe,0x00,0xc2,0x55,0x01,
   0x18,0xd7,0x3e,0x00,0x32,0x10,0x00,0x3e,0x00,0xcd,0xb6,0x01,
   0x3e,0x01,0xcd,0xb6,0x01,0x3e,0x02,0xcd,0xb6,0x01,0x3e,0x03,
   0xcd,0xb6,0x01,0xc3,0x58,0x00,0x3e,0x00,0x32,0x11,0x00,0x3a,
   0x20,0x00,0xe6,0x03,0xfe,0x00,0x20,0x06,0xdd,0x21,0x50,0x05,
   0x18,0x18,0xfe,0x01,0x20,0x06,0xdd,0x21,0x90,0x05,0x18,0x0e,
   0xfe,0x02,0x20,0x06,0xdd,0x21,0xd0,0x05,0x18,0x04,0xdd,0x21,
   0x10,0x06,0xfd,0x21,0x20,0x00,0xcd,0x6c,0x01,0xc3,0x60,0x00,
   0xe6,0x1f,0x5f,0x3e,0x00,0xcb,0x23,0xcb,0x23,0xcb,0x23,0xcb,
   0x23,0xce,0x00,0xcb,0x27,0xcb,0x23,0xce,0x00,0x57,0x21,0x50,
   0x06,0x19,0xe5,0xdd,0xe1,0xfd,0x21,0x20,0x00,0x3e,0x00,0xfd,
   0x46,0x00,0xdd,0x70,0x00,0xdd,0x23,0xfd,0x23,0x3c,0xfe,0x13,
   0x20,0xf1,0x3e,0x00,0x32,0x12,0x00,0xc3,0x68,0x00,0xe6,0x1f,
   0x5f,0x3e,0x00,0xcb,0x23,0xcb,0x23,0xcb,0x23,0xcb,0x23,0xce,
   0x00,0xcb,0x27,0xcb,0x23,0xce,0x00,0x57,0x21,0x50,0x06,0x19,
   0xe5,0xfd,0xe1,0xfd,0x7e,0x00,0xe6,0x03,0xfe,0x00,0x20,0x06,
   0xdd,0x21,0x50,0x05,0x18,0x18,0xfe,0x01,0x20,0x06,0xdd,0x21,
   0x90,0x05,0x18,0x0e,0xfe,0x02,0x20,0x06,0xdd,0x21,0xd0,0x05,
   0x18,0x04,0xdd,0x21,0x10,0x06,0xcd,0x6c,0x01,0x3e,0x00,0x32,
   0x13,0x00,0xc3,0x70,0x00,0x3e,0x00,0xcd,0xfa,0x04,0x3e,0x01,
   0xcd,0xfa,0x04,0x3e,0x02,0xcd,0xfa,0x04,0x3e,0x03,0xcd,0xfa,
   0x04,0xc3,0x78,0x00,0xdd,0xe5,0x3e,0x00,0xfd,0x46,0x00,0xdd,
   0x70,0x00,0xfd,0x23,0xdd,0x23,0x3c,0xfe,0x13,0x20,0xf1,0xdd,
   0xe1,0x3e,0x00,0xdd,0x77,0x13,0xdd,0x77,0x14,0xdd,0x77,0x15,
   0xdd,0x7e,0x0d,0xdd,0x77,0x19,0xdd,0x7e,0x0e,0xdd,0x77,0x1c,
   0xdd,0x7e,0x03,0xdd,0x77,0x17,0xdd,0x7e,0x04,0xdd,0x77,0x18,
   0xdd,0x7e,0x05,0xdd,0x77,0x1a,0xdd,0x7e,0x06,0xdd,0x77,0x1b,
   0x3e,0x01,0xdd,0x77,0x16,0xc9,0xfe,0x00,0x20,0x0e,0xfd,0x21,
   0x50,0x05,0x16,0x40,0x1e,0x70,0xdd,0x21,0x00,0x40,0x18,0x30,
   0xfe,0x01,0x20,0x0e,0xfd,0x21,0x90,0x05,0x16,0x00,0x1e,0x10,
   0xdd,0x21,0x01,0x40,0x18,0x1e,0xfe,0x02,0x20,0x0e,0xfd,0x21,
   0xd0,0x05,0x16,0x20,0x1e,0x30,0xdd,0x21,0x01,0x40,0x18,0x0c,
   0xfd,0x21,0x10,0x06,0x16,0x40,0x1e,0x50,0xdd,0x21,0x01,0x40,
   0xfd,0x7e,0x16,0xfe,0x00,0xca,0xdc,0x02,0xfd,0x7e,0x00,0xe6,
   0x03,0xfe,0x00,0x20,0x14,0xfd,0x7e,0x00,0xfe,0x00,0x20,0x04,
   0x3e,0x03,0x18,0x02,0x3e,0x07,0xf6,0x60,0xf6,0x80,0xdd,0x77,
   0x00,0xfd,0x7e,0x0d,0x47,0x3e,0x0f,0x90,0xf6,0x80,0xb3,0xdd,
   0x77,0x00,0xfd,0x4e,0x03,0x79,0xe6,0x0f,0xf6,0x80,0xb2,0xdd,
   0x77,0x00,0xfd,0x7e,0x04,0xe6,0x03,0x47,0xfd,0x7e,0x03,0xcb,
   0x3f,0xcb,0x38,0x30,0x02,0xf6,0x80,0xcb,0x3f,0xcb,0x38,0x30,
   0x02,0xf6,0x80,0xcb,0x3f,0xcb,0x3f,0xdd,0x77,0x00,0xfd,0x7e,
   0x07,0xfe,0x00,0x28,0x33,0x47,0xfd,0x7e,0x14,0x3c,0xfd,0x77,
   0x14,0xb8,0x20,0x28,0x3e,0x00,0xfd,0x77,0x14,0xfd,0x7e,0x08,
   0xfe,0x00,0x28,0x05,0xcd,0x24,0x03,0x18,0x17,0xfd,0x4e,0x05,
   0xfd,0x46,0x06,0xfd,0x6e,0x03,0xfd,0x66,0x04,0x09,0x7c,0xe6,
   0x03,0x67,0xfd,0x75,0x03,0xfd,0x74,0x04,0xfd,0x7e,0x0f,0xfe,
   0x00,0x28,0x29,0x47,0xfd,0x7e,0x15,0x3c,0xfd,0x77,0x15,0xb8,
   0x20,0x1e,0x3e,0x00,0xfd,0x77,0x15,0xfd,0x7e,0x10,0xfe,0x00,
   0x28,0x05,0xcd,0x4f,0x04,0x18,0x0d,0xfd,0x7e,0x0d,0x47,0xfd,
   0x7e,0x0e,0x80,0xe6,0x0f,0xfd,0x77,0x0d,0xfd,0x7e,0x13,0x47,
   0xfd,0x7e,0x01,0xb8,0x20,0x0b,0xfd,0x7e,0x00,0xcd,0xfa,0x04,
   0xcd,0xdd,0x02,0x18,0x07,0xfd,0x7e,0x13,0x3c,0xfd,0x77,0x13,
   0xc9,0xfd,0x7e,0x02,0xfe,0x00,0x28,0x3f,0x3e,0x00,0xfd,0x77,
   0x13,0xfd,0x77,0x14,0xfd,0x77,0x15,0x3e,0x01,0xfd,0x77,0x16,
   0xfd,0x7e,0x19,0xfd,0x77,0x0d,0xfd,0x7e,0x1c,0xfd,0x77,0x0e,
   0xfd,0x7e,0x17,0xfd,0x77,0x03,0xfd,0x7e,0x18,0xfd,0x77,0x04,
   0xfd,0x7e,0x1a,0xfd,0x77,0x05,0xfd,0x7e,0x1b,0xfd,0x77,0x06,
   0xfd,0x7e,0x02,0xfe,0xff,0x28,0x04,0x3d,0xfd,0x77,0x02,0xc9,
   0xdd,0xe5,0xe5,0xd5,0xc5,0xfd,0x7e,0x06,0xe6,0x02,0xfe,0x00,
   0xc2,0xb6,0x03,0xfd,0x6e,0x0b,0xfd,0x66,0x0c,0xfd,0x4e,0x03,
   0xfd,0x46,0x04,0x37,0x3f,0xed,0x42,0xe5,0xdd,0xe1,0xfd,0x4e,
   0x05,0xfd,0x46,0x06,0x37,0x3f,0xed,0x42,0x30,0x4e,0xfd,0x6e,
   0x05,0xfd,0x66,0x06,0xdd,0xe5,0xc1,0x37,0x3f,0xed,0x42,0xfd,
   0x7e,0x08,0xfe,0x01,0x20,0x12,0xe5,0xc1,0xfd,0x6e,0x09,0xfd,
   0x66,0x0a,0x09,0xfd,0x75,0x03,0xfd,0x74,0x04,0xc3,0x41,0x04,
   0xe5,0xc1,0xfd,0x6e,0x0b,0xfd,0x66,0x0c,0x37,0x3f,0xed,0x42,
   0xfd,0x75,0x03,0xfd,0x74,0x04,0x21,0x00,0x04,0xfd,0x4e,0x05,
   0xfd,0x46,0x06,0x37,0x3f,0xed,0x42,0xfd,0x75,0x05,0xfd,0x74,
   0x06,0xc3,0x41,0x04,0xfd,0x6e,0x03,0xfd,0x66,0x04,0xfd,0x4e,
   0x05,0xfd,0x46,0x06,0x09,0xfd,0x75,0x03,0xfd,0x74,0x04,0xc3,
   0x41,0x04,0xfd,0x6e,0x03,0xfd,0x66,0x04,0xfd,0x4e,0x09,0xfd,
   0x46,0x0a,0x37,0x3f,0xed,0x42,0xe5,0xdd,0xe1,0x21,0x00,0x04,
   0xfd,0x4e,0x05,0xfd,0x46,0x06,0x37,0x3f,0xed,0x42,0xe5,0xd1,
   0xdd,0xe5,0xe1,0xd5,0xc1,0x37,0x3f,0xed,0x42,0x30,0x48,0xd5,
   0xe1,0xdd,0xe5,0xc1,0x37,0x3f,0xed,0x42,0xfd,0x7e,0x08,0xfe,
   0x01,0x20,0x15,0xe5,0xc1,0xfd,0x6e,0x0b,0xfd,0x66,0x0c,0x37,
   0x3f,0xed,0x42,0xfd,0x75,0x03,0xfd,0x74,0x04,0xc3,0x41,0x04,
   0xfd,0x4e,0x09,0xfd,0x46,0x0a,0x09,0xfd,0x75,0x03,0xfd,0x74,
   0x04,0x21,0x00,0x04,0xfd,0x4e,0x05,0xfd,0x46,0x06,0x37,0x3f,
   0xed,0x42,0xfd,0x75,0x05,0xfd,0x74,0x06,0xc3,0x41,0x04,0xfd,
   0x6e,0x03,0xfd,0x66,0x04,0xfd,0x4e,0x05,0xfd,0x46,0x06,0x09,
   0xfd,0x75,0x03,0xfd,0x74,0x04,0xc3,0x41,0x04,0xfd,0x7e,0x04,
   0xe6,0x03,0xfd,0x77,0x04,0xc1,0xd1,0xe1,0xdd,0xe1,0xc9,0xd5,
   0xc5,0xfd,0x7e,0x0e,0xe6,0x08,0xfe,0x00,0xc2,0xa2,0x04,0xfd,
   0x7e,0x12,0xfd,0x4e,0x0d,0x91,0x47,0xfd,0x4e,0x0e,0x37,0x3f,
   0x99,0x30,0x2a,0xfd,0x7e,0x0e,0x48,0x91,0x57,0xfd,0x7e,0x10,
   0xfe,0x01,0x20,0x0a,0xfd,0x7e,0x11,0x82,0xfd,0x77,0x0d,0xc3,
   0xef,0x04,0xfd,0x7e,0x12,0x92,0xfd,0x77,0x0d,0x3e,0x10,0xfd,
   0x4e,0x0e,0x91,0xfd,0x77,0x0e,0xc3,0xef,0x04,0xfd,0x7e,0x0d,
   0xfd,0x4e,0x0e,0x81,0xfd,0x77,0x0d,0xc3,0xef,0x04,0xfd,0x7e,
   0x0d,0xfd,0x4e,0x11,0x91,0x47,0x3e,0x10,0xfd,0x4e,0x0e,0x91,
   0x57,0x4a,0x78,0x37,0x3f,0x99,0x30,0x2a,0x7a,0x48,0x91,0x5f,
   0xfd,0x7e,0x10,0xfe,0x01,0x20,0x0b,0xfd,0x7e,0x12,0x4b,0x91,
   0xfd,0x77,0x0d,0xc3,0xef,0x04,0xfd,0x7e,0x11,0x4b,0x81,0xfd,
   0x77,0x0d,0x3e,0x10,0xfd,0x4e,0x0e,0x91,0xfd,0x77,0x0e,0xc3,
   0xef,0x04,0xfd,0x7e,0x0d,0xfd,0x4e,0x0e,0x81,0xfd,0x77,0x0d,
   0xc3,0xef,0x04,0xfd,0x7e,0x0d,0xe6,0x0f,0xfd,0x77,0x0d,0xc1,
   0xd1,0xc9,0xdd,0xe5,0xfd,0xe5,0xd5,0xc5,0xe6,0x03,0xfe,0x00,
   0x20,0x0c,0xdd,0x21,0x00,0x40,0x06,0x70,0xfd,0x21,0x50,0x05,
   0x18,0x2a,0xfe,0x01,0x20,0x0c,0xdd,0x21,0x01,0x40,0x06,0x10,
   0xfd,0x21,0x90,0x05,0x18,0x1a,0xfe,0x02,0x20,0x0c,0xdd,0x21,
   0x01,0x40,0x06,0x30,0xfd,0x21,0xd0,0x05,0x18,0x0a,0xdd,0x21,
   0x01,0x40,0x06,0x50,0xfd,0x21,0x10,0x06,0x78,0xf6,0x80,0xf6,
   0x0f,0xdd,0x77,0x00,0x3e,0x00,0xfd,0x77,0x16,0xc1,0xd1,0xfd,
   0xe1,0xdd,0xe1,0xc9
};


void BlockCopy(u8 * Dest, const u8 * Source, u16 n)
{
   u8 *p1, *p2;

   p1 = Dest;
   p2 = (u8 *)Source;
   
   while (n--)
	  *p1++ = *p2++;
}



//////////////////////////////////////////////////////////////////////////////
// InstallSoundDriver
// To be called once at the start of the code. It copies the Z80 pre
// assembled SFX driver into the Z80 memory space.
//////////////////////////////////////////////////////////////////////////////
void InstallSoundDriver(void)
{
   u8 * pZ80RAM = Z80_RAM;
   u8 i;

   // first stop the z80
   SOUNDCPU_CTRL = 0xAAAA;

   // copy driver
   BlockCopy(pZ80RAM, SFXDriver_Z80, sizeof(SFXDriver_Z80));

   // restart z80
   SOUNDCPU_CTRL = 0x5555;

   // allows the VBI interrupt to start pulsing the Z80
   SFXInstalled = 1;
}


//////////////////////////////////////////////////////////////////////////////
// InstallSounds
// Copies the user defined sounds to the Z80 memory
//////////////////////////////////////////////////////////////////////////////
void InstallSounds(const SOUNDEFFECT SoundData[], u8 NumSounds)
{
   u8 Preset;
   u8 i;
   u8 * SFXBuff1 = (u8 *)0x7020;
   u8 * SFXBuff2 = (u8 *)0x7023;
   
   for (Preset = 0; Preset < NumSounds; Preset++)
   {
	   // copy SFX
	  u8 * SData = (u8 *)&SoundData[Preset];

	  // this is yuk but I couldn't find a way to stop the C compiler
	  // doing word alignment on the SoundEffects structure. This
	  // removes the excess padding.
	  BlockCopy(SFXBuff1, SData, 3);
	  SData+=4;
	  BlockCopy(SFXBuff2, SData, sizeof(SOUNDEFFECT)-4);

	  // instruct driver to store it
	  SFXLoadPreset = Preset+1;

	   // wait for completion
	  while(SFXLoadPreset)
		 ;
   }
}

//////////////////////////////////////////////////////////////////////////////
// PlaySound
// Plays one of the user defined sounds
// Inputs:
//          SoundNumber - the sound preset to play (range 1 to 31)
//
//////////////////////////////////////////////////////////////////////////////
void PlaySound(u8 SoundNumber)
{
   SFXPlayPreset = SoundNumber;
}

//////////////////////////////////////////////////////////////////////////////
// StopAllSound
// Call this to stop the sound playing in all NGPC sound channels
//////////////////////////////////////////////////////////////////////////////
void StopAllSound(void)
{
   SFXStopAll = 1;
}



#define RAND_MAX 32767
volatile u32 RAND_RandomData;

//////////////////////////////////////////////////////////////////////////////
// SeedRandom
// Unfortunately we need a pseudo random number in order to seed our
// pseudo random number generator otherwise the numbers would always
// come out the same.
// To solve this I use VBCounter that is incremented every VB
// Interrupt. If this function was called at initialisation the same
// problem would arrise as VBCounter will have always hit the same
// value.
// Call SeedRandom just after the user has pressed fire/start from the
// title screen of your game.
//////////////////////////////////////////////////////////////////////////////
void SeedRandom(void)
{
   RAND_RandomData = VBCounter;
   GetRandom(100);
}



// helper function as there is no 32bit multiply natively supported by
// the compiler :-0
s32 Multiply32bit(s32 Value1, s32 Value2)
{
   __ASM("  LD      XWA, (XSP+4)");
   __ASM("  LD      XBC, (XSP+8)");
   __ASM("  LD		HL,QWA");
   __ASM("  MUL		XHL,BC");
   __ASM("  LD		DE,QBC");
   __ASM("  MUL		XDE,WA");
   __ASM("  ADD		XHL,XDE");
   __ASM("  LD		QHL,HL");
   __ASM("  LD		HL,0");
   __ASM("  MUL		XWA,BC");
   __ASM("  ADD		XHL,XWA");

   return __XHL;
}

//////////////////////////////////////////////////////////////////////////////
// GetRandom
// Returns a pseudo random number in the range of 0 to the specified
// input value.
// Inputs:
//     Value = the upper limit for the random number (must be less than 32767)
//////////////////////////////////////////////////////////////////////////////
u16 GetRandom(u16 Value)
{
   RAND_RandomData = Multiply32bit(RAND_RandomData,20077);
   RAND_RandomData += 12345;
   
   return (Multiply32bit(((RAND_RandomData >> 16) & RAND_MAX), Value) >> 15);
}


// ----------------------------------------------
// QRandom Numbers - Chris Ahchay
// ----------------------------------------------
//Most of these aren't used... I'll get the correctly shaped tuit one of these days....
const unsigned char QuickRandomNumbers[1024]={63, 254, 102, 147, 252, 158, 83, 102, 90, 162, 89, 211, 146, 181,
33, 188, 239, 26, 52, 113, 168, 22, 140, 238, 236, 139, 163, 63, 92, 140, 49, 59, 196, 187, 155, 179, 77, 194,
203, 66, 226, 67, 239, 131, 193, 41, 115, 237, 253, 231, 230, 26, 201, 211, 37, 120, 217, 200, 147, 142, 88,
251, 156, 158, 223, 88, 255, 196, 160, 75, 19, 82, 24, 129, 212, 130, 118, 66, 20, 176, 87, 69, 79, 213, 195,
71, 109, 232, 48, 169, 93, 43, 206, 8, 52, 207, 92, 6, 47, 123, 153, 158, 256, 219, 247, 0, 48, 27, 140, 194,
74, 196, 206, 104, 78, 142, 13, 82, 65, 5, 220, 215, 72, 235, 122, 240, 166, 14, 149, 116, 233, 254, 159, 164,
52, 236, 252, 231, 84, 189, 158, 45, 1, 75, 130, 202, 89, 229, 136, 194, 61, 253, 91, 19, 1, 101, 34, 80, 74,
114, 243, 79, 172, 125, 101, 91, 42, 176, 31, 15, 171, 213, 127, 132, 256, 168, 190, 42, 100, 80, 78, 17, 104,
98, 142, 245, 158, 95, 111, 155, 131, 132, 220, 38, 158, 217, 186, 12, 29, 246, 31, 157, 19, 197, 228, 237, 248,
89, 62, 232, 247, 167, 92, 127, 100, 168, 200, 50, 113, 248, 144, 132, 91, 216, 57, 132, 140, 13, 251, 214, 30,
177, 252, 61, 1, 125, 90, 84, 161, 118, 71, 146, 74, 2, 27, 168, 158, 93, 199, 78, 201, 181, 229, 132, 226, 178,
24, 206, 158, 153, 29, 87, 93, 210, 226, 149, 206, 133, 69, 163, 53, 250, 195, 132, 5, 131, 219, 213, 35, 72, 150,
107, 26, 146, 64, 30, 39, 223, 105, 172, 109, 245, 11, 12, 20, 33, 223, 13, 40, 94, 104, 218, 238, 230, 26, 196,
255, 229, 107, 40, 155, 35, 218, 178, 115, 154, 120, 86, 163, 120, 235, 94, 44, 201, 164, 8, 80, 250, 25, 30, 256,
84, 24, 36, 177, 26, 180, 208, 190, 255, 143, 66, 34, 135, 185, 187, 248, 172, 242, 186, 177, 232, 180, 30, 206,
210, 214, 224, 152, 176, 13, 89, 30, 242, 156, 31, 138, 74, 239, 43, 151, 179, 60, 24, 186, 81, 141, 106, 133, 131,
58, 31, 221, 164, 141, 175, 57, 204, 63, 82, 52, 161, 106, 93, 124, 69, 17, 79, 113, 139, 195, 57, 103, 45, 225, 45,
51, 59, 251, 28, 30, 38, 204, 237, 198, 63, 217, 106, 109, 109, 63, 30, 114, 179, 117, 139, 102, 209, 186, 104, 202,
165, 165, 125, 222, 116, 161, 219, 179, 37, 156, 115, 4, 80, 10, 195, 214, 219, 211, 211, 49, 197, 151, 159, 35, 155,
49, 120, 157, 165, 182, 99, 189, 18, 204, 204, 218, 21, 239, 214, 90, 24, 69, 186, 218, 44, 154, 142, 149, 230, 253,
16, 98, 46, 163, 121, 25, 9, 61, 95, 225, 150, 149, 22, 199, 253, 19, 139, 246, 167, 28, 183, 238, 126, 37, 231, 66,
77, 21, 117, 40, 227, 155, 236, 62, 244, 227, 14, 244, 172, 71, 149, 174, 4, 205, 9, 18, 123, 175, 37, 147, 255, 27,
128, 228, 36, 70, 152, 60, 1, 13, 43, 7, 67, 131, 178, 8, 203, 67, 183, 66, 221, 120, 208, 48, 45, 197, 99, 159, 169,
91, 20, 219, 82, 245, 56, 127, 147, 136, 80, 201, 201, 161, 24, 212, 200, 253, 53, 141, 188, 247, 62, 147, 185, 212,
36, 253, 175, 238, 205, 147, 20, 154, 160, 74, 180, 169, 37, 85, 97, 12, 169, 195, 114, 215, 219, 83, 251, 25, 248,
140, 200, 179, 26, 175, 128, 136, 219, 182, 86, 215, 30, 71, 250, 159, 206, 7, 79, 173, 200, 194, 212, 244, 14, 49,
42, 241, 157, 239, 70, 63, 84, 85, 44, 190, 9, 18, 209, 24, 246, 169, 176, 232, 194, 54, 136, 132, 84, 148, 199, 12,
95, 155, 38, 124, 236, 88, 141, 222, 196, 221, 181, 162, 12, 154, 146, 122, 189, 138, 142, 137, 188, 48, 15, 161, 16,
52, 70, 24, 63, 56, 99, 216, 174, 95, 3, 59, 247, 235, 160, 8, 239, 57, 241, 58, 36, 163, 19, 24, 244, 80, 213, 93,
208, 223, 150, 59, 234, 212, 169, 65, 130, 250, 212, 49, 160, 114, 127, 202, 57, 244, 210, 252, 109, 191, 60, 102,
9, 51, 223, 28, 62, 30, 219, 224, 198, 37, 102, 167, 254, 111, 32, 33, 62, 212, 144, 128, 81, 97, 62, 63, 106, 48,
8, 55, 84, 134, 248,112, 211, 134, 3, 53, 35, 143, 58, 159, 219, 114, 56, 142, 100, 100, 97, 136, 216, 202, 217, 122,
233, 47, 81, 29, 10, 18, 179, 180, 44, 236, 186, 102, 4, 186, 83, 225, 130, 100, 179, 166, 190, 252, 39, 8, 85, 86,
226, 174, 158, 116, 214, 50, 187, 39, 40, 227, 176, 125, 79, 129, 85, 236, 165, 26, 97, 127, 117, 190, 161, 100, 183,
123, 27, 116, 86, 149, 226, 76, 178, 12, 154, 67, 67, 89, 228, 180, 206, 216, 74, 97, 222, 9, 159, 19, 130, 191, 120,
94, 7, 93, 233, 43, 251, 151, 148, 55, 28, 79, 43, 33, 8, 139, 243, 171, 189, 104, 19, 95, 252, 123, 130, 69, 16, 84,
191, 48, 159, 77, 196, 246, 34, 217, 153, 152, 216, 53, 36, 153, 205, 246, 52, 142, 115, 244, 45, 120, 171, 205, 219,
229, 179, 239, 93, 130, 126, 244, 19, 189, 184, 34, 247, 101, 176, 76, 94, 118, 56, 19, 240, 20, 197, 143, 219, 38,
42, 158, 9, 223, 173, 149, 211, 54, 255, 136, 11, 194, 26, 172, 73, 109, 14, 178, 9, 114, 121, 157, 117, 119, 91, 248,
29, 156, 194, 35, 156, 158, 8, 18, 170, 243, 145, 51, 93, 128, 92, 237, 192, 77, 238, 250, 173, 109, 41, 51, 43, 80,
241, 28, 172, 46, 99, 183, 35, 133, 170, 9, 120, 192, 44, 110, 182, 135, 193, 45, 161, 105, 151, 191, 170, 226, 53,
70, 31, 182, 2};

void InitialiseQRandom()
{
   RandomNumberCounter=0;
}

unsigned char QRandom()
{
    //Um. RandomNumberCounter is an unsigned char...
    //which means that this routine will never get beyond the first
    //255 elements of the random number array?
    //Oh well. Should be good enough for my uses anyway...
   return QuickRandomNumbers[RandomNumberCounter++];
}


//////////////////////////////////////////////////////////////////////////////
// Sine and Cosine
//////////////////////////////////////////////////////////////////////////////
const signed char SineTable256[256]=
{
   0,	3,	6,	9,	12,	16,	19,	22,	25,	28,	31,	34,	37,	40,	43,	46,	49,	51,	54,	57,	60,	63,	65,	68,	71,	73,	76,	78,	81,	83,	85,	88,	90,	92,	94,	96,	98,	100,	102,	104,	106,	107,	109,	111,	112,	113,	115,	116,	117,	118,	120,	121,	122,	122,	123,	124,	125,	125,	126,	126,	126,	127,	127,	127,	127,
   127,	127,	127,	126,	126,	126,	125,	125,	124,	123,	122,	122,	121,	120,	118,	117,	116,	115,	113,	112,	111,	109,	107,	106,	104,	102,	100,	98,	96,	94,	92,	90,	88,	85,	83,	81,	78,	76,	73,	71,	68,	65,	63,	60,	57,	54,	51,	49,	46,	43,	40,	37,	34,	31,	28,	25,	22,	19,	16,	12,	9,	6,	3,
   0,	-3,	-6,	-9,	-12,	-16,	-19,	-22,	-25,	-28,	-31,	-34,	-37,	-40,	-43,	-46,	-49,	-51,	-54,	-57,	-60,	-63,	-65,	-68,	-71,	-73,	-76,	-78,	-81,	-83,	-85,	-88,	-90,	-92,	-94,	-96,	-98,	-100,	-102,	-104,	-106,	-107,	-109,	-111,	-112,	-113,	-115,	-116,	-117,	-118,	-120,	-121,	-122,	-122,	-123,	-124,	-125,	-125,	-126,	-126,	-126,	-127,	-127,	-127,	-127,	-127,
   -127,	-127,	-126,	-126,	-126,	-125,	-125,	-124,	-123,	-122,	-122,	-121,	-120,	-118,	-117,	-116,	-115,	-113,	-112,	-111,	-109,	-107,	-106,	-104,	-102,	-100,	-98,	-96,	-94,	-92,	-90,	-88,	-85,	-83,	-81,	-78,	-76,	-73,	-71,	-68,	-65,	-63,	-60,	-57,	-54,	-51,	-49,	-46,	-43,	-40,	-37,	-34,	-31,	-28,	-25,	-22,	-19,	-16,	-12,	-9,	-6,	-3
};

signed char Sin(unsigned char dAngle)
{
   return SineTable256[dAngle];
}

signed char Cos(unsigned char dAngle)
{
   return SineTable256[dAngle+64];
}



//////////////////////////////////////////////////////////////////////////////
// Time and Alarm functions
//////////////////////////////////////////////////////////////////////////////
const char DayNames[7][10] = { 
   "Sunday", "Monday", "Tuesday", "Wednesday", 
   "Thursday", "Friday", "Saturday"
};


void GetTime(TIME * pTime)
{
   __asm(" ld rw3, 2");		// VECT_RTCGET
   __asm(" ld xde, (xsp+4)");	// get ptr off stack
   __asm(" ld xhl3, xde");	// pass in this register

   // do the system call
   __asm(" ldf 3");
   __asm(" add w,w");
   __asm(" add w,w");
   __asm(" ld xix, 0xfffe00");
   __asm(" ld xix,(xix+w)");
   __asm(" call xix");
}

void SetWake(ALARM * pAlarm)
{
// sets system to wakeup if powered off
// pass in a pointer to a alarm structure

   __asm(" ld rw3, 11");	// VECT_ALARMDOWNSET
   __asm(" ld xiy, (xsp+4)");	// get ptr
   __asm(" ldf 3");		// params go in reg bank 3
                // could probably do a 32 bit copy
                // to xbc but this looks better
   __asm(" ld h, (xiy +)");	// day of month
   __asm(" ld qc, h");
   __asm(" ld b, (xiy +)");	// hour
   __asm(" ld c, (xiy +)");	// min

   // do the system call
   __asm(" add w,w");
   __asm(" add w,w");
   __asm(" ld xix,0xfffe00");
   __asm(" ld xix,(xix+w)");
   __asm(" call xix");

   // returns success in A3 someday
//   __asm(" ld (xiy), a");
}

void SetAlarm(ALARM * pAlarm)
{
// sets an alarm while powered up
// I don't see much use in this one, but...
// need to setup interrupt first to catch this one
// and do something with it
// pass in a pointer to a alarm structure

   __asm(" ld rw3, 9");		// VECT_ALARMSET
   __asm(" ld xiy, (xsp+4)");	// get ptr
   __asm(" ldf 3");		// params go in reg bank 3
                // could probably do a 32 bit copy
                // to xbc but this looks better
   __asm(" ld h, (xiy +)");	// day of month
   __asm(" ld qc, h");
   __asm(" ld b, (xiy +)");	// hour
   __asm(" ld c, (xiy +)");	// min

   // do the system call
   __asm(" add w,w");
   __asm(" add w,w");
   __asm(" ld xix,0xfffe00");
   __asm(" ld xix,(xix+w)");
   __asm(" call xix");

// return success in A3 someday
//   __asm(" ld (xiy+3), a3");
}


void CpuSpeed(u8 spd)
{
// changes system clock speed
// pass 0 (full) to 4 (slowest)


   __asm("ld rw3, 1");		// VECT_CLOCKGEARSET
   __asm("ld xde, (xsp+4)");	// get spd off stack
   __asm("ld b, e");
   __asm("ld c, 0");		// no speedup on joypad

   // do the system call
   __asm(" ldf 3");
   __asm(" add w,w");
   __asm(" add w,w");
   __asm(" ld xix,0xfffe00");
   __asm(" ld xix,(xix+w)");
   __asm(" call xix");
};

void ResumeOff(void)
{
// turns off system after setting resume bit
// this skips the intro and can be detected to
// resume the game where it left off if programmed
// properly

   USR_ANSWER |= 128;
   SysShutdown();
};


//////////////////////////////////////////////////////////////////////////////
// NeoTracker integration
//////////////////////////////////////////////////////////////////////////////

// the pre-assembled Z80 NeoTracker driver
const unsigned char NeoTracker_Driver[] = {
	0xf3,0x31,0xc0,0x00,0xc3,0xe6,0x01,0x00,
	0xe5,0xcd,0xc2,0x08,0xe1,0xc9,0x00,0x00,
	0x7d,0x2f,0x6f,0x7c,0x2f,0x67,0x23,0xc9,
	0x5d,0x54,0x13,0xaf,0x77,0xed,0xb0,0xc9,
	0x21,0xdc,0x00,0x7e,0xd6,0x04,0xc9,0x00,
	0x85,0x6f,0xd0,0x24,0xc9,0x00,0x00,0x00,
	0xc3,0x8f,0x04,0xca,0x31,0x0e,0x00,0x00,
	0xf3,0x08,0x3c,0xd3,0xff,0x08,0xfb,0xed,
	0x4d,0x11,0x00,0x00,0x48,0x42,0xb7,0xc8,
	0x1f,0x30,0x03,0xeb,0x09,0xeb,0xcb,0x21,
	0xcb,0x10,0x18,0xf2,0x47,0x3a,0xe5,0x01,
	0xcd,0x41,0x00,0xeb,0x29,0x29,0x7c,0xc3,
	0x75,0x04,0x00,0x00,0x00,0x00,0xf5,0xe5,
	0xd5,0x21,0xd1,0x00,0xcd,0xc7,0x04,0x11,
	0x00,0x80,0x1a,0x77,0xee,0xff,0x12,0xb7,
	0xcc,0x2c,0x06,0xd1,0xe1,0xf1,0xed,0x45,
	0x4b,0x31,0x0b,0x30,0x28,0x78,0x0a,0xff,
	0x20,0x20,0x20,0x42,0xff,0xff,0x00,0xb5,
	0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xed,0x56,
	0x2e,0x01,0x7d,0xcf,0x7d,0xc6,0x20,0x6f,
	0x07,0x30,0xf7,0x21,0xc0,0x00,0x01,0x25,
	0x01,0xdf,0x21,0xd7,0x00,0x36,0x78,0x08,
	0xaf,0x08,0xd3,0xff,0xfb,0x32,0x00,0xc0,
	0x21,0xd0,0x00,0x3a,0xd1,0x00,0xbe,0x28,
	0x07,0xcd,0xc7,0x04,0x7e,0xcd,0xfa,0x04,
	0xf3,0x08,0x4f,0x08,0xfb,0x21,0xde,0x00,
	0x7e,0x91,0x28,0xe4,0xed,0x44,0x71,0x23,
	0x77,0x21,0xd4,0x00,0x1e,0x00,0x01,0xd2,
	0x00,0xcd,0x4a,0x02,0x21,0xd7,0x00,0x1e,
	0x04,0x01,0xd3,0x00,0xcd,0x4a,0x02,0x3a,
	0xdf,0x00,0x47,0xaf,0xcf,0xcd,0xe3,0x05,
	0x18,0xbe,0x0a,0xb7,0xc8,0x4f,0xe5,0xd9,
	0xe1,0x3a,0xdf,0x00,0x87,0x87,0x46,0xcd,
	0x41,0x00,0x23,0x4e,0x23,0x46,0xeb,0x09,
	0x01,0xe2,0x04,0xb7,0xed,0x42,0x30,0x01,
	0x09,0xeb,0x72,0x2b,0x73,0xd9,0xd8,0x21,
	0xa5,0x02,0x22,0x85,0x02,0x69,0x60,0x06,
	0x04,0x16,0x01,0xed,0x53,0xdc,0x00,0xd5,
	0xc5,0xcd,0x96,0x02,0xcd,0xa5,0x02,0xc1,
	0xd1,0x1c,0xcb,0x02,0x10,0xed,0xc9,0x21,
	0x16,0x06,0x1e,0x00,0x18,0xdc,0x3a,0xdc,
	0x00,0x06,0x20,0xcd,0x41,0x00,0xfd,0x21,
	0xe5,0x00,0xfd,0x19,0xc9,0xcb,0x1d,0xd0,
	0xcd,0xb3,0x02,0xe5,0xcd,0xf9,0x02,0xe1,
	0xaf,0x18,0x02,0x3e,0x01,0xd9,0xfd,0xcb,
	0x08,0x46,0x21,0x03,0x00,0x20,0x03,0x21,
	0x0c,0x00,0xfd,0xe5,0xd1,0x19,0x11,0xe0,
	0x00,0xb7,0x20,0x01,0xeb,0x01,0x05,0x00,
	0xed,0xb0,0xd9,0xc9,0x57,0x2a,0xe2,0x00,
	0x19,0xeb,0xcd,0xdc,0x05,0xcd,0xb0,0x02,
	0xfd,0xcb,0x08,0x86,0xaf,0xfd,0x77,0x01,
	0x18,0xc9,0x21,0x00,0x00,0x22,0xe0,0x00,
	0xcd,0xb0,0x02,0xfd,0xcb,0x08,0xc6,0x18,
	0xeb,0xfd,0xe5,0xe1,0x7e,0xb7,0x28,0x08,
	0x35,0x20,0x05,0x0e,0x00,0xcd,0x7b,0x03,
	0x23,0x7e,0xd6,0x01,0x77,0xd0,0x3e,0x01,
	0xf7,0x4f,0xe5,0x21,0xe4,0x00,0xcd,0x79,
	0x04,0x47,0xcb,0x19,0x30,0x07,0xcb,0x42,
	0x28,0x03,0x06,0x00,0x70,0xe1,0x70,0xe5,
	0xcd,0x7f,0x03,0xe1,0x18,0xdb,0x3e,0x02,
	0xf7,0xfd,0x46,0x18,0xfe,0x03,0x28,0x22,
	0x57,0x3e,0x0f,0xa0,0x4f,0x3e,0xf0,0xa0,
	0x47,0xcb,0x1a,0x38,0x08,0x3e,0x03,0xf7,
	0x87,0x87,0x87,0x87,0x47,0xcb,0x1a,0x79,
	0x38,0x03,0x3e,0x04,0xf7,0xb0,0x47,0xfd,
	0x70,0x18,0x3e,0x01,0xf7,0xfd,0x4e,0x11,
	0x20,0x07,0x3e,0x04,0xf7,0xfd,0x77,0x11,
	0x4f,0xfd,0xe5,0xe1,0x3e,0x02,0xef,0xcd,
	0x79,0x04,0xfd,0x86,0x00,0xfd,0x77,0x00,
	0x79,0xb7,0xc8,0x16,0x03,0x18,0x51,0x30,
	0xad,0x3e,0x04,0xf7,0xfe,0x0f,0xca,0x01,
	0x04,0x0e,0xff,0x57,0x87,0x82,0x21,0xe6,
	0x03,0xef,0x7e,0xb7,0x28,0x01,0xf7,0x47,
	0x23,0x7a,0xfe,0x04,0x38,0x06,0x7e,0x23,
	0x66,0x6f,0x78,0xe9,0x56,0x23,0x7e,0xb7,
	0x28,0x26,0xfd,0xe5,0xe1,0xef,0xcb,0x79,
	0x28,0x1d,0x7a,0xfe,0x02,0x20,0x0d,0x4a,
	0xeb,0xfd,0x6e,0x0a,0xfd,0x66,0x0b,0x78,
	0xef,0x46,0xeb,0x51,0x3a,0xda,0x00,0xb7,
	0x28,0x04,0x3e,0x05,0xba,0xc8,0x70,0x46,
	0xd9,0xe7,0x30,0x0b,0x3a,0xd3,0x00,0x23,
	0xa6,0x28,0x02,0xd9,0xc9,0x2b,0x7e,0xd9,
	0x0f,0x0f,0x0f,0xb2,0xcf,0xc9,0x05,0x02,
	0x12,0x04,0x05,0x15,0x04,0x06,0x16,0x07,
	0x04,0x17,0x05,0x2e,0x04,0x00,0x41,0x04,
	0x08,0x72,0x04,0x0b,0xd4,0x02,0x00,0xea,
	0x02,0xfd,0xe5,0xcd,0x0b,0x04,0xfd,0xe1,
	0xf1,0xf1,0xc9,0xcd,0x16,0x06,0x3a,0xd2,
	0x00,0x21,0xdd,0x00,0xa6,0xc8,0xe7,0xcd,
	0x99,0x02,0x21,0xe7,0x03,0x06,0x04,0x0e,
	0x00,0xe5,0xc5,0xcd,0xa4,0x03,0xc1,0xe1,
	0x3e,0x03,0xef,0x10,0xf4,0xc9,0x4f,0xb7,
	0x20,0x02,0x0e,0xff,0xcd,0x63,0x04,0x34,
	0xeb,0x71,0x23,0x11,0xe2,0x00,0xeb,0x18,
	0x18,0xcd,0x63,0x04,0xeb,0x01,0xfd,0xff,
	0x09,0x7e,0xfe,0xff,0x28,0x06,0x3d,0x20,
	0x03,0xeb,0x35,0xc9,0x77,0x23,0x11,0xe2,
	0x00,0x01,0x02,0x00,0xed,0xb0,0xaf,0x32,
	0xe1,0x00,0xc9,0xfd,0xe5,0xe1,0x3e,0x19,
	0xef,0x5d,0x54,0x7e,0x87,0x86,0x23,0xef,
	0xeb,0xc9,0x32,0xe5,0x01,0x32,0xd4,0x00,
	0xc9,0x3e,0x01,0xf7,0x28,0x04,0x7e,0x16,
	0x01,0xc9,0x3c,0xf7,0x28,0x02,0x3e,0x04,
	0xc6,0x04,0xf7,0x77,0x16,0x00,0xc9,0xfe,
	0x09,0x38,0x0a,0xf5,0x3e,0x08,0xcd,0x9d,
	0x04,0x5f,0xf1,0xd6,0x08,0xd9,0xe6,0x0f,
	0x47,0x2a,0xe0,0x00,0x7c,0x26,0x00,0xb7,
	0x20,0x0d,0xed,0x5b,0xe2,0x00,0x1a,0x13,
	0xed,0x53,0xe2,0x00,0x6f,0x3e,0x08,0xcb,
	0x25,0xcb,0x14,0x3d,0x10,0xe9,0x54,0x67,
	0x22,0xe0,0x00,0x7a,0xd9,0xb7,0xc9,0x7e,
	0x34,0xe6,0x0f,0x21,0xc0,0x00,0xef,0xc9,
	0x3e,0xff,0x1e,0x04,0x21,0x84,0x05,0x4f,
	0x3a,0xd3,0x00,0x47,0xc3,0x72,0x02,0xfe,
	0x86,0x28,0xed,0x0f,0xe6,0x1f,0x21,0xbb,
	0x0a,0x87,0xef,0x7e,0x23,0x46,0x21,0xf6,
	0x04,0x87,0xef,0xc3,0x9e,0x03,0xfa,0x05,
	0x54,0x00,0xb7,0xfa,0xdf,0x04,0x0f,0x38,
	0x10,0x0f,0x30,0xce,0x2a,0xb9,0x0a,0x11,
	0xd3,0x00,0xe6,0x1f,0x47,0x0e,0x04,0x18,
	0x11,0xf5,0xcd,0x8f,0x02,0xf1,0x0f,0xd0,
	0x47,0x9f,0x2a,0xb7,0x0a,0x11,0xd2,0x00,
	0x0e,0x00,0x32,0xc3,0x05,0xed,0x53,0x99,
	0x05,0x78,0xe6,0x1f,0x28,0x0b,0x47,0x5e,
	0x23,0x7e,0xe6,0x3f,0x57,0x2b,0x19,0x10,
	0xf6,0x23,0x7e,0xe6,0xc0,0x07,0x07,0x23,
	0x3c,0x47,0xc5,0xe5,0xdd,0xe1,0x3e,0x02,
	0xcd,0x41,0x00,0xdd,0x19,0xc1,0x5e,0x23,
	0x56,0x3e,0x3f,0xa2,0x57,0xc5,0xd5,0x7e,
	0x23,0xe6,0xc0,0x07,0x07,0xf5,0x81,0xcd,
	0x99,0x02,0xf1,0x0e,0x01,0x47,0xb7,0x28,
	0x04,0xcb,0x01,0x10,0xfc,0xd1,0xcd,0x96,
	0x05,0xc1,0x10,0xda,0x3e,0x04,0xb9,0xc8,
	0x21,0xd4,0x00,0x36,0x78,0x23,0xaf,0x77,
	0x23,0x77,0x18,0x76,0xcb,0x1c,0xd0,0x7d,
	0x07,0x38,0x05,0xfd,0x7e,0x09,0xbd,0xc0,
	0xe5,0xcd,0x0b,0x04,0xe1,0xc9,0x79,0xd9,
	0x21,0xd2,0x00,0xa6,0xd9,0x28,0x0a,0x3a,
	0xc3,0x05,0xfd,0xbe,0x09,0x28,0x02,0x30,
	0x2b,0x79,0xd9,0xb6,0x77,0xfd,0xe5,0xe1,
	0x01,0x1f,0x00,0xdf,0xd9,0xeb,0x19,0xeb,
	0xfd,0x73,0x05,0xfd,0x72,0x06,0xfd,0xcb,
	0x08,0xc6,0x3e,0x00,0xfd,0x77,0x09,0x07,
	0xd4,0xd5,0x05,0xdd,0xe5,0xd1,0xfd,0x73,
	0x0a,0xfd,0x72,0x0b,0xc9,0x11,0x33,0x00,
	0xfd,0xcb,0x08,0x86,0xfd,0x73,0x0e,0xfd,
	0x72,0x0f,0xc9,0x21,0xdb,0x00,0x7e,0xb7,
	0xc8,0x35,0xc0,0x2b,0x7e,0x23,0x77,0x21,
	0x05,0x06,0xcd,0x92,0x02,0x3a,0xd2,0x00,
	0xb7,0xc0,0x06,0x11,0xcd,0x41,0x00,0x53,
	0xed,0x53,0xda,0x00,0xc9,0xfd,0x7e,0x15,
	0x3c,0xfe,0x0f,0x28,0x09,0xfd,0x77,0x15,
	0x47,0x16,0x05,0xc3,0xd0,0x03,0x16,0x01,
	0xcd,0xd0,0x03,0xd9,0xe7,0x11,0xd2,0x00,
	0x38,0x01,0x13,0x23,0x7e,0xeb,0xee,0xff,
	0xa6,0x77,0xd9,0xc9,0x21,0x00,0x00,0xe5,
	0xed,0x45,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0xbc,0x05,0x6a,
	0x05,0x1c,0x05,0xd2,0x04,0x8c,0x04,0x4c,
	0x04,0x0e,0x04,0xd4,0x03,0x9c,0x03,0x69,
	0x03,0x38,0x03,0x09,0x03,0x0f,0x06,0x05,
	0x04,0x03,0x02,0x01,0x00,0x00,0x10,0x20,
	0x30,0x40,0x50,0x60,0xf0,0x00,0x01,0x02,
	0x04,0x0c,0x00,0x0c,0x0f,0x0d,0x0b,0x09,
	0x07,0x05,0xff,0xff,0x43,0xd7,0x03,0xb5,
	0x36,0x98,0xdd,0x21,0x34,0x06,0xe6,0x60,
	0x32,0x32,0x06,0xc8,0x11,0x22,0x00,0xdd,
	0x19,0xd6,0x20,0x20,0xfa,0xc9,0xcd,0x5e,
	0x07,0x5c,0x87,0x21,0xbd,0x06,0xef,0x7b,
	0x5e,0x23,0x56,0xb7,0xc8,0xcb,0x3a,0xcb,
	0x1b,0x3d,0x20,0xf9,0xc9,0xed,0x44,0x6f,
	0x87,0x9f,0x67,0xd8,0x29,0xc9,0x3a,0x32,
	0x06,0xfe,0x60,0xc0,0xf6,0x83,0xcb,0x61,
	0x28,0x02,0xf6,0x04,0x32,0x00,0x40,0xc9,
	0xfe,0x0f,0xd8,0x3e,0x0f,0xc9,0x21,0x00,
	0x00,0x3e,0x10,0x29,0xeb,0xed,0x6a,0xeb,
	0x30,0x04,0x09,0x30,0x01,0x1c,0x3d,0x20,
	0xf2,0x53,0x5c,0xc9,0x1a,0x13,0x6f,0x26,
	0x00,0x29,0x29,0x29,0x29,0xe6,0x0f,0x6f,
	0xc9,0x1a,0x13,0x4f,0x07,0x07,0x07,0xe6,
	0x07,0xef,0x7e,0xc9,0xd5,0xdd,0x5e,0x00,
	0xdd,0x56,0x01,0xdd,0x6e,0x02,0xdd,0x66,
	0x03,0x3a,0xbc,0x06,0x19,0x3d,0x20,0xfc,
	0x7c,0xaa,0xfa,0xb2,0x07,0x7c,0xb7,0xfa,
	0x98,0x07,0xb9,0x38,0x1d,0x79,0x18,0x06,
	0x79,0xed,0x44,0xbc,0x38,0x14,0x67,0x2e,
	0x00,0xcb,0x48,0x20,0x0c,0xcb,0x50,0x20,
	0x09,0xeb,0xd7,0xeb,0xcd,0x0e,0x08,0x18,
	0x01,0xd7,0xd1,0xdd,0x75,0x02,0xdd,0x74,
	0x03,0x7c,0xc9,0x21,0x00,0x00,0x87,0xc8,
	0xf5,0xc5,0xd5,0xeb,0x0f,0x2f,0x3c,0x6f,
	0xe6,0x03,0x5f,0x7d,0x0f,0x0f,0xe6,0x07,
	0x28,0x04,0xc6,0x02,0x18,0x03,0x7b,0x3d,
	0x5a,0x21,0xf2,0x06,0x19,0x19,0x5e,0x23,
	0x56,0xb7,0x28,0x07,0xcb,0x3a,0xcb,0x1b,
	0x3d,0x20,0xf9,0x06,0x00,0xcd,0x46,0x07,
	0xeb,0xd1,0xc1,0xf1,0xd0,0xd7,0xc9,0xaf,
	0xdd,0x77,0x03,0xdd,0x77,0x02,0xdd,0x7e,
	0x07,0xdd,0x77,0x08,0xdd,0x7e,0x06,0xdd,
	0x4e,0x04,0xcd,0xbb,0x07,0xeb,0xdd,0x73,
	0x00,0xdd,0x72,0x01,0xc9,0xcb,0x48,0x20,
	0x01,0x4f,0x57,0xdd,0x7e,0x01,0xe6,0x80,
	0xdd,0xb6,0x06,0xcd,0x0a,0x08,0x7a,0xcb,
	0x48,0xc8,0x4f,0xc9,0xdd,0xe5,0xdd,0x19,
	0x5f,0xdd,0x7e,0x06,0xb7,0x28,0x0a,0xdd,
	0x7e,0x08,0xb7,0x28,0x08,0x3d,0xdd,0x77,
	0x08,0xaf,0x4f,0x18,0x20,0x7b,0xcb,0x40,
	0x28,0x05,0xcd,0x15,0x08,0x18,0x03,0xdd,
	0x4e,0x04,0xc5,0xdd,0x46,0x05,0xcd,0x74,
	0x07,0xcb,0x40,0x28,0x07,0xb7,0x79,0xf2,
	0x64,0x08,0xed,0x44,0xc1,0xdd,0x56,0x04,
	0xdd,0x6e,0x05,0xdd,0xe1,0xcb,0x5d,0xc8,
	0x67,0x7a,0xcb,0x7d,0x20,0x02,0xed,0x44,
	0x84,0xc9,0x11,0x06,0x00,0xdd,0xe5,0xdd,
	0x19,0x01,0x3f,0x00,0xcd,0x74,0x07,0xdd,
	0xe1,0xa7,0xf2,0x8e,0x08,0xaf,0x47,0xdd,
	0x7e,0x0a,0x3d,0x28,0x09,0x3d,0x28,0x0c,
	0x78,0xb7,0x28,0x1b,0x18,0x1c,0x78,0xb9,
	0x38,0x18,0x18,0x07,0xdd,0x4e,0x0f,0x79,
	0xb8,0x38,0x0f,0x41,0xdd,0x36,0x08,0x00,
	0xdd,0x70,0x09,0xdd,0x7e,0x0a,0x3c,0xcd,
	0x90,0x09,0x3e,0x3f,0x90,0x0f,0x0f,0xe6,
	0x0f,0xc9,0x6f,0xcd,0xfa,0x06,0x7d,0xe6,
	0x0f,0x87,0x21,0xd3,0x08,0xef,0x7e,0x23,
	0x66,0x6f,0xe9,0x24,0x0a,0x14,0x0a,0xe1,
	0x08,0x3e,0x09,0xc1,0x09,0x10,0x0a,0x06,
	0x0a,0x2a,0xb5,0x0a,0x78,0x87,0x87,0x87,
	0x80,0xef,0xeb,0x21,0xea,0x06,0xcd,0x69,
	0x07,0xdd,0x77,0x05,0x79,0xe6,0x0f,0xdd,
	0x77,0x0b,0xcd,0x5c,0x07,0xdd,0x74,0x0c,
	0x87,0x87,0xdd,0x77,0x0f,0xcd,0x5c,0x07,
	0xdd,0x74,0x0d,0xdd,0x75,0x0e,0xcd,0x2e,
	0x07,0x01,0x10,0x00,0xcd,0x1a,0x09,0x01,
	0x09,0x00,0xdd,0x09,0x1a,0xe6,0x80,0x47,
	0x1a,0x13,0xe6,0x7f,0xdd,0x77,0x04,0x21,
	0xe5,0x06,0xcd,0x69,0x07,0xb0,0xdd,0x77,
	0x05,0x79,0xe6,0x1f,0xb0,0xdd,0x77,0x06,
	0x1a,0x13,0xdd,0x77,0x07,0xc9,0x3e,0x0f,
	0x91,0x4f,0xfe,0x0f,0x20,0x09,0xdd,0x7e,
	0x0a,0xb7,0xc8,0x3e,0x04,0x18,0x41,0xdd,
	0x71,0x02,0x3a,0x32,0x06,0xfe,0x60,0x78,
	0x20,0x02,0xc6,0x30,0xdd,0x77,0x00,0xdd,
	0x36,0x01,0x00,0xcd,0x0e,0x07,0xcd,0xe3,
	0x09,0xdd,0xe5,0x11,0x10,0x00,0xdd,0x19,
	0xcd,0xf7,0x07,0x11,0x09,0x00,0xdd,0x19,
	0xcd,0xf7,0x07,0xdd,0xcb,0x05,0x56,0xdd,
	0xe1,0x28,0x04,0xdd,0xcb,0x05,0xf6,0xaf,
	0xdd,0x77,0x08,0xdd,0x77,0x09,0x3e,0x01,
	0xdd,0x77,0x0a,0xb7,0xc8,0x5f,0xdd,0xe5,
	0x16,0x00,0xdd,0x19,0xfe,0x02,0x38,0x02,
	0x16,0x80,0xdd,0x7e,0x0a,0xdd,0xe1,0x87,
	0x20,0x09,0x7b,0x1c,0xfe,0x02,0x28,0xe6,
	0xaf,0x18,0x02,0x3c,0xb2,0x0e,0x3f,0xcd,
	0xbb,0x07,0xdd,0x75,0x06,0xdd,0x74,0x07,
	0xc9,0x78,0xd6,0x40,0x87,0xdd,0x77,0x01,
	0x21,0x00,0x00,0xb7,0x28,0x05,0xeb,0xcd,
	0x25,0x07,0x19,0x4d,0x44,0xdd,0x7e,0x00,
	0xcd,0x0e,0x07,0x78,0xb1,0x28,0x04,0x04,
	0xcd,0x46,0x07,0x21,0x01,0x40,0x3a,0x32,
	0x06,0xfe,0x60,0x20,0x03,0x2d,0x3e,0x40,
	0x4f,0x7b,0xe6,0x0f,0xf6,0x80,0xb1,0xeb,
	0x29,0x29,0x29,0x29,0xeb,0x5f,0x7a,0xe6,
	0x7f,0xf3,0x73,0x77,0xfb,0xc9,0x21,0xd5,
	0x06,0x78,0xef,0x7e,0xdd,0x77,0x04,0xc9,
	0xdd,0x70,0x03,0xc9,0xdd,0xe5,0xe1,0x01,
	0x21,0x00,0xdf,0x01,0x0f,0x0f,0xdd,0x71,
	0x02,0xc3,0xa4,0x0a,0x78,0x32,0xbc,0x06,
	0xaf,0xf5,0xcd,0xfa,0x06,0xcd,0x38,0x0a,
	0xf1,0xc6,0x20,0xfe,0x80,0x38,0xf2,0xc9,
	0xdd,0x7e,0x0a,0xb7,0xc8,0xdd,0x46,0x05,
	0x04,0x05,0x28,0x36,0xcb,0x80,0x11,0x10,
	0x00,0xcd,0x2c,0x08,0xed,0x44,0x81,0x32,
	0x33,0x06,0xdd,0x46,0x05,0x11,0x19,0x00,
	0xcd,0x2c,0x08,0xcb,0x40,0x28,0x08,0xf5,
	0xed,0x44,0x81,0x32,0x33,0x06,0xf1,0xcb,
	0x50,0x28,0x0f,0xcd,0x25,0x07,0xcb,0x70,
	0x28,0x02,0x29,0x29,0xdd,0x7e,0x01,0xcd,
	0xcb,0x09,0xcd,0x7a,0x08,0xdd,0x86,0x02,
	0xdd,0x86,0x03,0xdd,0xcb,0x05,0x5e,0x28,
	0x09,0x47,0x3a,0x33,0x06,0xcd,0x5e,0x07,
	0x7c,0x80,0x47,0xdd,0x7e,0x04,0xcd,0x5e,
	0x07,0x80,0xcd,0x40,0x07,0x4f,0x7c,0x80,
	0xcd,0x40,0x07,0x47,0x21,0x00,0x40,0x3a,
	0x32,0x06,0xf6,0x90,0xb0,0x47,0xe6,0xf0,
	0xb1,0x77,0x2c,0x70,0xc9
};

//////////////////////////////////////////////////////////////////////////////
// NeoTracker_InstallDriver
// To be called once at the start of the code. It copies the Z80 pre
// assembled SFX driver into the Z80 memory space.
//////////////////////////////////////////////////////////////////////////////
void NeoTracker_InstallDriver(void)
{
   u8 * pZ80RAM = Z80_RAM;
   u8 i;

   // first stop the z80
   SOUNDCPU_CTRL = 0xAAAA;

   // copy driver
   BlockCopy(pZ80RAM, NeoTracker_Driver, sizeof(NeoTracker_Driver));

   // restart z80
   SOUNDCPU_CTRL = 0x5555;

   TRUN &= 0xF7;    // turn off timer 3
   T23MOD &= 0x33;  // set timers to 8 bit
   T23MOD |= 4;     // t3 internal clock = t1
   TFFCR &= 0x0f;   // clear timer 3 settings
   TFFCR |= 0xb0;   // clear flip flop and flip on timer 3
   TREG3 = 0x62;    // set comparitor
   TRUN = 0x88;     // turn timer 3 back on
   
}

// sends a group the z80 driver
#define NEOTRACKER_ACK  (*(u8*)0x70DE)
void NeoTracker_SendGroup(const u8 theGroup[], const int Len)
{
   u8 i;
   volatile u8 ack;
   u8 * pZ80Mem = Z80_RAM;
   pZ80Mem += sizeof(NeoTracker_Driver);
   
	// stop any sound playing
   NeoTracker_StopAll();
		 
	// wait a while for the driver to ack the stop command
   for (i = 0; i < 100; i++)
   {
	  volatile u8 Value = Z80_COMM;
	  if (Value == 0)
		 break;
   }
   // if no ack then reinstall the driver
   if (i == 100)
	  NeoTracker_InstallDriver();

   // copy the group
   BlockCopy(pZ80Mem, theGroup, Len);

   // wait for acknowledge
   ack = NEOTRACKER_ACK;
   do
   {
	  Sleep(1);
   } while (ack == NEOTRACKER_ACK);
}


// plays specified music
void NeoTracker_PlayMusic(const int Number)
{
   u8 RealNum = (Number << 2) | 3;
   // instruct driver
   Z80_COMM = RealNum;
   Z80_NMI = 1;
}

// plays a SFX - may temp interrupt the music
void NeoTracker_PlaySFX(const int Number)
{
   u8 RealNum = (Number << 2) | 2;
   // instruct driver
   Z80_COMM = RealNum;
   Z80_NMI = 1;
}

// Stops any sound playing (music and SFX)
void NeoTracker_StopAll(void)
{
   // instruct driver
   Z80_COMM = 0xff;
   Z80_NMI = 1;
}


//////////////////////////////////////////////////////////////////////////////
// Bitmap Mode support - Chris Ahchay
//////////////////////////////////////////////////////////////////////////////

//Bit of a rethink here. This is a vanilla bitmap function.
//It works, up to a point, but there are a ton of problems with it
//for real world use:
//1, You can only create a single bitmap.
//2. It's slow (probably not much I can do about that)
//3. Blt'ing bits is painful.
//So...
//Retool this to...

//Pass a memory address into the functions. This replaces the top/left
//co-ordinates.
//Everything works relative to this address, rather than fixed to BitmapTileBase
//Write a third function (or, more accurately, redo CopySingleTile()) to blt
//one bitmap into another (i.e. move from an array to the screen)

//The bitmap passed must be big enough to store (Width*Height)+8) elements
//defined as (to store a 160 by 96 size bitmap (160/8)*(96/8)=240 for the
//bitmap tiles and eight elements for the bitmap properties.
//u16 Bitmap[241][8]
//or
//u16 Bitmap[1928]
//Doesn't matter which, but the first is the easiest to use IMHO...

//Bitmap masks
const unsigned short BitmapMask[8] = {0x3fff, 0xcfff, 0xf3ff, 0xfcff, 0xff3f, 0xffcf, 0xfff3, 0xfffc};

const unsigned short BitmapNewMask[4][8] =
{
   {0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
   {0x4000, 0x1000, 0x0400, 0x0100, 0x0040, 0x0010, 0x0004, 0x0001},
   {0x8000, 0x2000, 0x0800, 0x0200, 0x0080, 0x0020, 0x0008, 0x0002},
   {0xc000, 0x3000, 0x0c00, 0x0300, 0x00c0, 0x0030, 0x000c, 0x0003}
};

const unsigned short OffsetValues[8]={0x1, 0x4, 0x10, 0x40, 0x100, 0x400, 0x1000, 0x4000};


void CreateBitmap(u16 * BitmapAddress, u8 Width, u8 Height)
{
	u8 iLoopX;
	u8 iLoopY;
	u16 * MyAddress= BitmapAddress;
	u8 iFillLoop;

	//Create an array of tiles for a bitmap operation thang

	//Save the bitmap dimensions for later
	//Use the *first* element of the bitmap array to store the properties of the
	//bitmap.
	MyAddress[0]=Width;
	MyAddress[1]=Height;
	//Can put other stuff in here
	//MyAddress[2]=BackgroundColour
	//MyAddress[3]=Colour 1
	//MyAddress[4]=Colour 2
	//MyAddress[5]=Colour 3
	//MyAddress[6]=X Position
	//MyAddress[7]=Y Position
	MyAddress+=8;

	for (iLoopX=0;iLoopX<(Width/8);iLoopX++)
	{
		for (iLoopY=0;iLoopY<(Height/8);iLoopY++)
		{
			//Copies a blank tile into the bitmap
			for (iFillLoop=0;iFillLoop<8;iFillLoop++)
			{
				MyAddress++[0]=0;
			}

		}
	}
}

void SetPixel(u16 * BitmapAddress, u16 xPosition, u16 yPosition, u16 Colour)
{
	//Set the selected pixel in the bitmap
	u16 CurrentTile;
	//u16 TileRow;
	u16 * TileRam = BitmapAddress+8;

	//Trouble is. This needs to know the width and height of the bitmap (well, probably not the height)

	//Fist of all, identify the tile that the specified bitmap lives inside...
	//This can be derived from xPosition and yPosition

	CurrentTile=(xPosition/8)+((yPosition/8)*(BitmapAddress[0]/8));
	//For now, copy our "rubbish" tile into the tile space.
	//This is just to prove that I'm in the right general area...
	//CopySingleTile(RubbishTile, CustomTileBase+CurrentTile);

	//Having got the tile number. We now need to calculate the *new* tile
	//to put into here.

	//Basic method.
	//Get the existing tile into an unsigned short [8] Array
	//I don't need the whole tile. Just the row (yPosition % 8)
	//Grab a pointer to the relevant tile.
	//TileRam += (BitmapTileBase * 8);
	TileRam += (CurrentTile * 8)+(yPosition%8);
	//& Then... Grab the value held in this address...
	//TileRow = *TileRam;


	//Do clever stuff with AND's and OR's with the existing tile and the BitmapMask
	//array(s?)
	//Yer basic method;
	//AND MaskRow[xPosition%8] TileRow
	//OR ColourRow[xPosition%8] TileRow
	//Where MaskRow & ColourRow are 16bit jobbies showing a "holey" bitmap and a "coloured" bitmap
	//respectively. (Load the values in bitmap.h into neotile to see what I mean)

	//Updated to work directly on the tileram. Slightly quicker...
	TileRam[0]=TileRam[0] & BitmapMask[xPosition%8];
	TileRam[0]=TileRam[0] | BitmapNewMask[Colour][xPosition%8];

	//Put the tile back...
	//TileRam[0] = TileRow;

}

void QuickSetPixel(u16 * BitmapAddress, u16 xPosition, u16 yPosition, u16 Colour)
{
	//To be used when we can guarantee that only a single pixel in a tile row is affected.
	//Set the selected pixel in the bitmap
	u16 CurrentTile;
	//u16 TileRow;
	u16 * TileRam = BitmapAddress+8;

	//Trouble is. This needs to know the width and height of the bitmap (well, probably not the height)

	//Fist of all, identify the tile that the specified bitmap lives inside...
	//This can be derived from xPosition and yPosition

	CurrentTile=(xPosition/8)+((yPosition/8)*(BitmapAddress[0]/8));
	//For now, copy our "rubbish" tile into the tile space.
	//This is just to prove that I'm in the right general area...
	//CopySingleTile(RubbishTile, CustomTileBase+CurrentTile);

	//Having got the tile number. We now need to calculate the *new* tile
	//to put into here.

	//Basic method.
	//Get the existing tile into an unsigned short [8] Array
	//I don't need the whole tile. Just the row (yPosition % 8)
	//Grab a pointer to the relevant tile.
	//TileRam += (BitmapTileBase * 8);
	TileRam += (CurrentTile * 8)+(yPosition%8);
	//& Then... Grab the value held in this address...
	//TileRow = *TileRam;


	//Do clever stuff with AND's and OR's with the existing tile and the BitmapMask
	//array(s?)
	//Yer basic method;
	//AND MaskRow[xPosition%8] TileRow
	//OR ColourRow[xPosition%8] TileRow
	//Where MaskRow & ColourRow are 16bit jobbies showing a "holey" bitmap and a "coloured" bitmap
	//respectively. (Load the values in bitmap.h into neotile to see what I mean)

	//Updated to work directly on the tileram. Slightly quicker...
	//TileRam[0]=TileRam[0] & BitmapMask[xPosition%8];
	TileRam[0]=TileRam[0] = BitmapNewMask[Colour][xPosition%8];

	//Put the tile back...
	//TileRam[0] = TileRow;

}

//Don't worry about GetPixel() yet...
/*
u8 GetPixel(u16 xPosition, u16 yPosition)
{
	//Get the selected pixel in the bitmap

	//Sort of based on SetPixel() Let's get that working first..
	u16 CurrentTile;
	u16 TileRow;
	u16 * TileRam = TILE_RAM;
	u8 iReturn;
	u8 iOffset;
	u16 lHighBit;
	u16 lLowBit;
	//Fist of all, identify the tile that the specified bitmap lives inside...
	//This can be derived from xPosition and yPosition

	CurrentTile=(xPosition/8)+((yPosition/8)*(CurrentBitmapWidth/8));

	//Basic method.
	//Get the existing tile into an unsigned short [8] Array
	//I don't need the whole tile. Just the row (yPosition % 8)
	TileRam += (BitmapTileBase * 8);
	TileRam += (CurrentTile * 8)+(yPosition%8);

	//Fetch the actual value of the tile row...
	TileRow = *TileRam;

	//Work out which *bits* are used for this pixel
	iOffset=(7-(xPosition%8));
	lLowBit = OffsetValues[iOffset];
	lHighBit = (lLowBit * 2);
	//Right then. Are any of these set?
	iReturn=0;
	if ((TileRow & (lHighBit+lLowBit))==(lHighBit+lLowBit))
		iReturn=3;
	else if ((TileRow & (lHighBit))==(lHighBit))
		iReturn=2;
	else if ((TileRow & (lLowBit))==(lLowBit))
		iReturn=1;

	//Send something back...
	return iReturn;
}
*/

//Try it this way...
//Blts a single "bitmap" tile into TILE_RAM (one way?)
//Could we use CopySpriteTile()? Looks pretty similar to me.
void PutBitmapTile(u16 * BitmapAddress, u16 InputTile, u16 OutputTile)
{
	//A generic loop variable
	u16 i;
	//TileRam is a pointer to the beginning of tile memory
	u16 * TileRam = TILE_RAM;
	//theTiles is a pointer to the beginning of our Tiles array.
	//Copy a single tile into the specified tile number
	u16 * theTiles = (u16 *)BitmapAddress;

	TileRam += (OutputTile * 8);
	theTiles += (InputTile * 8);

	for (i = 0; i < 8; i ++)
	{
		//Copy the tile data
		TileRam[i] = *theTiles++;
	}
}

void CopyBitmap(u16 * BitmapAddress, u16 TileNumber)
{
	u8 iLoopX;
	u8 iLoopY;
	u8 TileWidth;
	u8 TileHeight;
	u16 * MyAddress= BitmapAddress+8;

	TileWidth=BitmapAddress[0]/8;
	TileHeight=BitmapAddress[1]/8;
	//Copy the bitmap to the screen.
	for (iLoopX=0;iLoopX<TileWidth;iLoopX++)
	{
		for (iLoopY=0;iLoopY<TileHeight;iLoopY++)
		{
			PutBitmapTile((u16 *)MyAddress, (iLoopY*TileWidth)+iLoopX, TileNumber+(iLoopY*TileWidth)+iLoopX);
		}
	}
}

void DrawBitmap(u16 * BitmapAddress, u8 xPosition, u8 yPosition, u16 TileNumber)
{
	u8 iLoopX;
	u8 iLoopY;
	u8 TileWidth;
	u8 TileHeight;

	TileWidth=BitmapAddress[0]>>3;
	TileHeight=BitmapAddress[0]>>3;

	//Copy the bitmap tiles into tile memory and blatt them onto the screen.
	for (iLoopX=0;iLoopX<TileWidth;iLoopX++)
	{
		for (iLoopY=0;iLoopY<TileHeight;iLoopY++)
		{
			PutTile(SCR_1_PLANE, 0, iLoopX+xPosition, iLoopY+yPosition, TileNumber+(iLoopY*TileWidth)+iLoopX);
		}
	}

}
void DrawLine(u16 * BitmapAddress, u8 x1, u8 y1, u8 x2, u8 y2, u8 Colour)
{
	//Stolen wholesale from gamedev...
	u8 xinc1;
	u8 xinc2;
	u8 yinc1;
	u8 yinc2;
	u8 den;
	u8 num;
	u8 numadd;
	u8 numpixels;
	u8 curpixel;
	u8 deltax;
	u8 deltay;
	u8 x;
	u8 y;

	//There appears to be an absence of an abs() function.
	if (x2>x1)
		deltax = x2 - x1;        // The difference between the x's
	else
		deltax = x1 - x2;
	if (y2>y1)
		deltay = y2 - y1;        // The difference between the y's
	else
		deltay = y1 - y2;

	x = x1;                       // Start x off at the first pixel
	y = y1;                       // Start y off at the first pixel

	if (x2 >= x1)                 // The x-values are increasing
	{
		xinc1 = 1;
		xinc2 = 1;
	}
	else                          // The x-values are decreasing
	{
		xinc1 = -1;
		xinc2 = -1;
	}

	if (y2 >= y1)                 // The y-values are increasing
	{
		yinc1 = 1;
		yinc2 = 1;
	}
	else                          // The y-values are decreasing
	{
		yinc1 = -1;
		yinc2 = -1;
	}

	if (deltax >= deltay)         // There is at least one x-value for every y-value
	{
		xinc1 = 0;                  // Don't change the x when numerator >= denominator
		yinc2 = 0;                  // Don't change the y for every iteration
		den = deltax;
		num = deltax / 2;
		numadd = deltay;
		numpixels = deltax;         // There are more x-values than y-values
	}
	else                          // There is at least one y-value for every x-value
	{
		xinc2 = 0;                  // Don't change the x for every iteration
		yinc1 = 0;                  // Don't change the y when numerator >= denominator
		den = deltay;
		num = deltay / 2;
		numadd = deltax;
		numpixels = deltay;         // There are more y-values than x-values
	}

	for (curpixel = 0; curpixel <= numpixels; curpixel++)
	{
		SetPixel((u16 *)BitmapAddress, x, y, Colour);             // Draw the current pixel
		num += numadd;              // Increase the numerator by the top of the fraction
		if (num >= den)             // Check if numerator >= denominator
		{
			num -= den;               // Calculate the new numerator value
			x += xinc1;               // Change the x as appropriate
			y += yinc1;               // Change the y as appropriate
		}
		x += xinc2;                 // Change the x as appropriate
		y += yinc2;                 // Change the y as appropriate
	}
}


//Two specific cases for vertical and horizontal lines
void DrawLineVertical(u16 * BitmapAddress, u16 x, u16 y1, u16 y2, u16 Colour)
{
	u16 y;
	s8 yinc;

	if (y1>y2)
		yinc=-1;
	else
		yinc=1;

	for (y=y1;y!=y2;y+=yinc)
	{
		SetPixel((u16 *)BitmapAddress, x, y, Colour);
	}
}


void DrawLineHorizontal(u16 * BitmapAddress, u16 x1, u16 x2, u16 y, u16 Colour)
{
	u16 x;
	s8 xinc;

	if (x1>x2)
		xinc=-1;
	else
		xinc=1;

	for (x=x1;x!=x2;x+=xinc)
	{
		SetPixel((u16 *)BitmapAddress, x, y, Colour);
	}
}


